Attribute VB_Name = "modFreeImage"
Option Explicit


'=============================================================================================================
'
' modFreeImage Module
' -------------------
'
' Created By  : Kevin Wilson
'               http://w3.one.net/~kevinw/VB   ( The VB Zone )
'
' Last Update : April 22, 2002
'
' VB Versions : 5.0 / 6.0
'
' Requires    : FreeImage.dll [v2.5.2 or better] (FreeImage library by Floris van den Berg - flvdberg@wxs.nl)
'               http://www.6ixsoft.com
'
' Description : This module wraps the exported functions and functionality of the FreeImage library
'               making usable to Visual Basic 5/6 programmers.
'
' Supported   : .BMP     [IN/OUT] Windows or OS/2 Bitmap
'               .WBMP    [IN/OUT] Wireless Bitmap - A bitmap type used by cell phones, etc.  Black & white.
'               .PBM     [IN/OUT] Portable Bitmap - Black & white ASCII based bitmap format
'               .PGM     [IN/OUT] Portable Greymap - Grayscale ASCII based bitmap format
'               .PPM     [IN/OUT] Portable Pixmap - High color ASCII based bitmap format
'               .PBMRAW  [IN/OUT] (Binary version of .PBM)
'               .PGMRAW  [IN/OUT] (Binary version of .PGM)
'               .PPMRAW  [IN/OUT] (Binary version of .PPM)
'               .ICO     [IN    ] Windows Icon
'               .PSD     [IN    ] Adobe Photoshop Document
'               .JPEG    [IN/OUT] Joint Photographic Experts Group (JFIF Compliant) - Popular high color bitmap format used on the WWW
'               .JBIG    [IN/OUT] Joint Bi-level Image experts Group.  Cousin of JPEG, but B&W (REQUIRES PLUG-IN)
'               .JNG     [IN    ] JPEG Network Graphics.  Cousin of MNG, but uses JPEG compression
'               .PNG     [IN/OUT] Portable Network Graphics - The answer to GIF when UNISYS became intrusive about their LZW compression algorithm
'               .MNG     [IN    ] Multiple Network Graphics - PNG's answer to animated GIF.
'               .PCX     [IN    ] Zsoft Paintbrush
'               .TIFF    [IN/OUT] Tag Image File Format - A widely used format for storing image data
'               .TARGA   [IN    ] Tagged Image File Format - Popular bitmap format on the Amiga and other computers
'               .KOALA   [IN    ] KOALA files were popular on the commodore 64. Added for nostalgic reasons
'               .PCD     [IN    ] Kodak PhotoCD. The PhotoCD format was developed by Kodak as an alternative to
'                                 analog photography. Natively supported by most DVD players and CD-I.
'               .RAS     [IN    ] Sun Raster File - Popular among Solaris computers
'               .IFF     [IN    ] Interchanged File Format - Designed by Electronic Arts as an Amiga image storage format
'               .LBM     [IN    ] LBM was created for the Deluxe Paint package, and is essentially the same as IFF. (still part of FreeImage.dll for backwards compatibility)
'               .CUT     [IN    ] Dr.Halo bitmap - Pretty popular some years ago
'
' Note        : Within this code you'll see the following return types:
'                 - FIBITMAP      = A handle to a Windows Device Independant Bitmap (DIB)
'                 - FIMULTIBITMAP = A handle to a "Multi-Page" bitmap (meaning an image that contains multiple
'                                   pages... like TIFF's sometimes are).  Refer to "FreeImage_LoadMultiPage"
'                 - DIB           = A handle to a Windows Device Independant Bitmap (DIB)
'                 - RGBQUAD       = A single RGBQUAD structure (type) represents the RED, GREE, & BLUE colors
'                                   make up a single color.  An array of RGBQUAD's is commonly used as a palette
'
' Example Use :
'
'  Option Explicit
'  Private Sub Form_Load()
'    Const TEST_FILE As String = "C:\TEST.BMP"
'    Dim ImageFormat As FREE_IMAGE_FORMAT
'    Dim DIB         As Long
'    Dim DIB_New     As Long
'    Dim hBITMAP     As Long
'    Me.AutoRedraw = True
'    Me.ScaleMode = vbPixels
'    Me.Visible = True
'    FreeImage_Initialise
'    ImageFormat = FreeImage_GetFileType(TEST_FILE, FileLen(TEST_FILE))
'    If ImageFormat <> FIF_UNKNOWN Then
'      DIB = FreeImage_Load(ImageFormat, TEST_FILE, 0)
'      If DIB <> 0 Then
'        If FreeImage_BMP_From_DIB(DIB, hBITMAP) = True Then
'          FreeImage_RenderBitmap hBITMAP, Me.hDC
'          If FreeImage_DIB_From_BMP(hBITMAP, DIB_New) = True Then
'            FreeImage_RenderDIB DIB_New, Me.hDC
'            FreeImage_Free DIB_New
'          End If
'          DeleteObject hBITMAP
'        End If
'        FreeImage_Unload DIB
'      End If
'    End If
'    FreeImage_DeInitialise
'  End Sub
'
'=============================================================================================================


'=============================================================================================================
' FreeImage 2
'
' Design and implementation by
' - Floris van den Berg (flvdberg@wxs.nl)
'
' Contributors:
' - Adam Gates (radad@xoasis.com)
' - Alex Kwak
' - Alexander Dymerets (sashad@te.net.ua)
' - Herv Drolon (drolon@iut.univ-lehavre.fr)
' - Jan L. Nauta (jln@magentammt.com)
' - Jani Kajala (janik@remedy.fi)
' - Juergen Riecker (j.riecker@gmx.de)
' - Luca Piergentili (l.pierge@terra.es)
' - Machiel ten Brinke (brinkem@uni-one.nl)
' - Markus Loibl (markus.loibl@epost.de)
' - Martin Weber (martweb@gmx.net)
' - Matthias Wandel (mwandel@rim.net)
'
' This file is part of FreeImage 2
'
' COVERED CODE IS PROVIDED UNDER THIS LICENSE ON AN "AS IS" BASIS, WITHOUT WARRANTY
' OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
' THAT THE COVERED CODE IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE
' OR NON-INFRINGING. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE COVERED
' CODE IS WITH YOU. SHOULD ANY COVERED CODE PROVE DEFECTIVE IN ANY RESPECT, YOU (NOT
' THE INITIAL DEVELOPER OR ANY OTHER CONTRIBUTOR) ASSUME THE COST OF ANY NECESSARY
' SERVICING, REPAIR OR CORRECTION. THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL
' PART OF THIS LICENSE. NO USE OF ANY COVERED CODE IS AUTHORIZED HEREUNDER EXCEPT UNDER
' THIS DISCLAIMER.
'
' Use at your own risk!
'=============================================================================================================


'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXX                       TYPE / ENUMERATION  DECLARATIONS                  XXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


'_____________________________________________________________________________________________________________
' Bitmap types
'

' Describes a color consisting of relative intensities of red, green, and blue.
Public Type RGBQUAD
  rgbBlue     As Byte '//BYTE  Specifies the intensity of blue in the color.
  rgbGreen    As Byte '//BYTE  Specifies the intensity of green in the color.
  rgbRed      As Byte '//BYTE  Specifies the intensity of red in the color.
  rgbReserved As Byte '//BYTE  Reserved; must be zero.
End Type

' Contains information about the dimensions and color format of a device-independent bitmap (DIB)
Public Type BITMAPINFOHEADER
  biSize          As Long    '//DWORD  Specifies the number of bytes required by the structure.
  biWidth         As Long    '//LONG   Specifies the width of the bitmap, in pixels.
  biHeight        As Long    '//LONG   Specifies the height of the bitmap, in pixels. If biHeight is positive, the bitmap is a bottom-up DIB and its origin is the lower left corner. If biHeight is negative, the bitmap is a top-down DIB and its origin is the upper left corner.
  biPlanes        As Integer '//WORD   Specifies the number of planes for the target device. This value must be set to 1.
  biBitCount      As Integer '//WORD   Specifies the number of bits per pixel. This value must be 1, 4, 8, 16, 24, or 32.
  biCompression   As Long    '//DWORD  Specifies the type of compression for a compressed bottom-up bitmap (top-down DIBs cannot be compressed). It can be one of the following values: BI_RGB, BI_RLE8, BI_RLE4, BI_BITFIELDS
  biSizeImage     As Long    '//DWORD  Specifies the size, in bytes, of the image. This may be set to 0 for BI_RGB bitmaps.
  biXPelsPerMeter As Long    '//LONG   Specifies the horizontal resolution, in pixels per meter, of the target device for the bitmap. An application can use this value to select a bitmap from a resource group that best matches the characteristics of the current device.
  biYPelsPerMeter As Long    '//LONG   Specifies the vertical resolution, in pixels per meter, of the target device for the bitmap.
  biClrUsed       As Long    '//DWORD  Specifies the number of color indices in the color table that are actually used by the bitmap. If this value is zero, the bitmap uses the maximum number of colors corresponding to the value of the biBitCount member for the compression mode specified by biCompression.
                             '         If biClrUsed is nonzero and the biBitCount member is less than 16, the biClrUsed member specifies the actual number of colors the graphics engine or device driver accesses. If biBitCount is 16 or greater, then biClrUsed member specifies the size of the color table used to optimize performance of Windows color palettes. If biBitCount equals 16 or 32, the optimal color palette starts immediately following the three doubleword masks.
                             '         If the bitmap is a packed bitmap (a bitmap in which the bitmap array immediately follows the BITMAPINFO header and which is referenced by a single pointer), the biClrUsed member must be either 0 or the actual size of the color table.
  biClrImportant  As Long    '//DWORD  Specifies the number of color indices that are considered important for displaying the bitmap. If this value is zero, all colors are important.
End Type

' Defines the dimensions and color information for a Windows device-independent bitmap (DIB)
Public Type BITMAPINFO
  bmiHeader   As BITMAPINFOHEADER '// BITMAPINFOHEADER  Specifies a BITMAPINFOHEADER structure that contains information about the dimensions and color format of a DIB.
  bmiColors() As RGBQUAD          '// RGBQUAD           Specifies an array of RGBQUAD or doubleword data types that define the colors in the bitmap.
End Type

' Defines the type, width, height, color format, and bit values of a bitmap.
Public Type BITMAP
  bmType       As Long    '// LONG   Specifies the bitmap type. This member must be zero.
  bmWidth      As Long    '// LONG   Specifies the width, in pixels, of the bitmap. The width must be greater than zero.
  bmHeight     As Long    '// LONG   Specifies the height, in pixels, of the bitmap. The height must be greater than zero.
  bmWidthBytes As Long    '// LONG   Specifies the number of bytes in each scan line. This value must be divisible by 2, because Windows assumes that the bit values of a bitmap form an array that is word aligned.
  bmPlanes     As Integer '// WORD   Specifies the count of color planes.
  bmBitsPixel  As Integer '// WORD   Specifies the number of bits required to indicate the color of a pixel.
  bmBits       As Long    '// LPVOID Points to the location of the bit values for the bitmap. The bmBits member must be a long pointer to an array of character (1-byte) values.
End Type

' FreeImageIO is a structure containing pointers to 4 functions: a read function, write function,
' seek function and tell function.  All these functions have to be implemented so that data is
' delivered.  The handle representing the data is made abstract as well and is named fi_handle.
Public Type FreeImageIO
  read_proc  As Long '// FI_ReadProc   // pointer to the function used to read data
  write_proc As Long '// FI_WriteProc  // pointer to the function used to write data
  seek_proc  As Long '// FI_SeekProc   // pointer to the function used to seek
  tell_proc  As Long '// FI_TellProc   // pointer to the function used to aquire the current position
End Type

'_____________________________________________________________________________________________________________
' Other Types
'
Public Type FreeImage
  Allocate_Proc                      As Long '// FI_AllocateProc
  Unload_Proc                        As Long '// FI_UnloadProc
  Free_Proc                          As Long '// FI_FreeProc
  Get_Colors_Used_Proc               As Long '// FI_GetColorsUsedProc
  Get_Bits_Proc                      As Long '// FI_GetBitsProc
  Get_Bits_row_col_Proc              As Long '// FI_GetBitsRowColProc
  Get_Scanline_Proc                  As Long '// FI_GetScanLineProc
  Get_BPP_Proc                       As Long '// FI_GetBPPProc
  Get_Width_Proc                     As Long '// FI_GetWidthProc
  Get_Height_Proc                    As Long '// FI_GetHeightProc
  Get_Line_Proc                      As Long '// FI_GetLineProc
  Get_Pitch_Proc                     As Long '// FI_GetPitchProc
  Get_DIB_Size_Proc                  As Long '// FI_GetDIBSizeProc
  Get_Palette_Proc                   As Long '// FI_GetPaletteProc
  Get_Dots_Per_Meter_X_Proc          As Long '// FI_GetDotsPerMeterXProc
  Get_Dots_Per_Meter_Y_Proc          As Long '// FI_GetDotsPerMeterYProc
  Get_Info_Header_Proc               As Long '// FI_GetInfoHeaderProc
  Get_Info_Proc                      As Long '// FI_GetInfoProc
  Get_Color_Type_Proc                As Long '// FI_GetColorTypeProc
  Get_Red_Mask_Proc                  As Long '// FI_GetRedMaskProc
  Get_Green_Mask_Proc                As Long '// FI_GetGreenMaskProc
  Get_Blue_Mask_Proc                 As Long '// FI_GetBlueMaskProc
  Get_Transparency_Count_Proc        As Long '// FI_GetTransparencyCountProc
  Get_Transparency_Table_Proc        As Long '// FI_GetTransparencyTableProc
  Set_Transparency_Table_Proc        As Long '// FI_SetTransparencyTableProc
  Is_Transparent_Proc                As Long '// FI_IsTransparentProc
  Set_Transparent_Proc               As Long '// FI_SetTransparentProc
  Output_Message_Proc                As Long '// FI_OutputMessageProc
  Convert_Line1to8_Proc              As Long '// FI_ConvertLine1To8Proc
  Convert_Line_4to8_Proc             As Long '// FI_ConvertLine4To8Proc
  Convert_Line_16to8_555_Proc        As Long '// FI_ConvertLine16To8_555Proc
  Convert_Line_16to8_565_Proc        As Long '// FI_ConvertLine16To8_565Proc
  Convert_Line_24to8_Proc            As Long '// FI_ConvertLine24To8Proc
  Convert_Line_32to8_Proc            As Long '// FI_ConvertLine32To8Proc
  Convert_Line_1to16_555_Proc        As Long '// FI_ConvertLine1To16_555Proc
  Convert_Line_4to16_555_Proc        As Long '// FI_ConvertLine4To16_555Proc
  Convert_Line_8to16_555_Proc        As Long '// FI_ConvertLine8To16_555Proc
  Convert_Line_16_565_to_16_555_Proc As Long '// FI_ConvertLine16_565_To16_555Proc
  Convert_Line_24to16_555_Proc       As Long '// FI_ConvertLine24To16_555Proc
  Convert_Line_32to16_555_Proc       As Long '// FI_ConvertLine32To16_555Proc
  Convert_Line_1to16_565_Proc        As Long '// FI_ConvertLine1To16_565Proc
  Convert_Line_4to16_565_Proc        As Long '// FI_ConvertLine4To16_565Proc
  Convert_Line_8to16_565_Proc        As Long '// FI_ConvertLine8To16_565Proc
  Convert_Line_16_555_to_16_565_Proc As Long '// FI_ConvertLine16_555_To16_565Proc
  Convert_Line_24to16_565_Proc       As Long '// FI_ConvertLine24To16_565Proc
  Convert_Line_32to16_565_Proc       As Long '// FI_ConvertLine32To16_565Proc
  Convert_Line_1to24_Proc            As Long '// FI_ConvertLine1To24Proc
  Convert_Line_4to24_Proc            As Long '// FI_ConvertLine4To24Proc
  Convert_Line_8to24_Proc            As Long '// FI_ConvertLine8To24Proc
  Convert_Line_16to24_555_Proc       As Long '// FI_ConvertLine16To24_555Proc
  Convert_Line_16to24_565_Proc       As Long '// FI_ConvertLine16To24_565Proc
  Convert_Line_32to24_Proc           As Long '// FI_ConvertLine32To24Proc
  Convert_Line_1to32_Proc            As Long '// FI_ConvertLine1To32Proc
  Convert_Line_4to32_Proc            As Long '// FI_ConvertLine4To32Proc
  Convert_Line_8to32_Proc            As Long '// FI_ConvertLine8To32Proc
  Convert_Line_16to32_555_Proc       As Long '// FI_ConvertLine16To32_555Proc
  Convert_Line_16to32_565_Proc       As Long '// FI_ConvertLine16To32_565Proc
  Convert_Line_24to32_Proc           As Long '// FI_ConvertLine24To32Proc
End Type

Public Type Plugin
  Format_Proc              As Long '// FI_FormatProc
  Description_Proc         As Long '// FI_DescriptionProc
  Extension_Proc           As Long '// FI_ExtensionListProc
  RegExpr_Proc             As Long '// FI_RegExprProc
  Open_Proc                As Long '// FI_OpenProc
  Close_Proc               As Long '// FI_CloseProc
  PageCount_Proc           As Long '// FI_PageCountProc
  PageCapability_Proc      As Long '// FI_PageCapabilityProc
  Load_Proc                As Long '// FI_LoadProc
  Save_Proc                As Long '// FI_SaveProc
  Validate_Proc            As Long '// FI_ValidateProc
  Mime_Proc                As Long '// FI_MimeProc
  Supports_Export_BPP_proc As Long '// FI_SupportsExportBPPProc
End Type

'_____________________________________________________________________________________________________________
' Important enums
'

Public Enum FREE_IMAGE_FORMAT
  FIF_UNKNOWN = -1  '// Unidentified bitmap type
  FIF_BMP = 0       '// Windows or OS/2 Bitmap File (*.BMP)
  FIF_ICO           '// Windows Icon (*.ICO)
  FIF_JPEG          '// Independent JPEG Group (*.JPG)
  FIF_JNG           '// JPEG Network Graphics (*.JNG)
  FIF_KOALA         '// Commodore 64 Koala format (*.KOA)
  FIF_LBM           '// Deluxe Paint LBM
  FIF_MNG           '// Multiple Network Graphics (*.MNG)
  FIF_PBM           '// Portable Bitmap (ASCII) (*.PBM)
  FIF_PBMRAW        '// Portable Bitmap (BINARY) (*.PBM)
  FIF_PCD           '// Kodak PhotoCD (*.PCD)
  FIF_PCX           '// PCX bitmap format (*.PCX)
  FIF_PGM           '// Portable Graymap (ASCII) (*.PGM)
  FIF_PGMRAW        '// Portable Graymap (BINARY) (*.PGM)
  FIF_PNG           '// Portable Network Graphics (*.PNG)
  FIF_PPM           '// Portable Pixelmap (ASCII) (*.PPM)
  FIF_PPMRAW        '// Portable Pixelmap (BINARY) (*.PPM)
  FIF_RAS           '// Sun Rasterfile (*.RAS)
  FIF_TARGA         '// Targa files (*.TGA)
  FIF_TIFF          '// Tagged Image File Format (*.TIFF)
  FIF_WBMP          '// Wireless Bitmap (*.WBMP)
  FIF_PSD           '// Photoshop (*.PSD)
  FIF_CUT           '// Dr. Halo (*.CUT)
  FIF_IFF = FIF_LBM '// Amiga IFF (*.IFF, *.LBM)
End Enum

Public Enum FREE_IMAGE_COLOR_TYPE
  FIC_MINISWHITE = 0 '// Monochrome bitmap: first palette entry is white
  FIC_MINISBLACK = 1 '// Monochrome bitmap: first palette entry is black
                     '   Palettized bitmap: grayscale palette
  FIC_RGB = 2        '// High-color bitmap (16, 24 or 32 bit)
  FIC_PALETTE = 3    '// Palettized bitmap (1, 4 or 8 bit)
  FIC_RGBALPHA = 4   '// High-color bitmap with an alpha channel (32 bit only)
End Enum

Public Enum FREE_IMAGE_QUANTIZE
  FIQ_WUQUANT = 0 '// Xiaolin Wu color quantization algorithm
  FIQ_NNQUANT = 1 '// NeuQuant neural-net quantization algorithm by Anthony Dekker
End Enum

Public Enum BOOL
  TRUE_ = 1  '// TRUE in C/C++ = 1  ... TRUE in Visual Basic = -1
  FALSE_ = 0 '// FALSE in C/C++ = 0 ... FALSE in Visual Basic = 0
End Enum


'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXX                          CONSTANTS  DECLARATIONS                        XXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


'_____________________________________________________________________________________________________________
' Load/Save flag constants
'
Public Const BMP_DEFAULT         As Long = 0     ' [In/Out] Default flag
Public Const CUT_DEFAULT         As Long = 0     ' [In/Out] Default flag
Public Const ICO_DEFAULT         As Long = 0     ' [In/Out] Default flag
Public Const ICO_FIRST           As Long = 0     ' [In    ] Loads the first bitmap in the icon
Public Const ICO_SECOND          As Long = 0     ' [In    ] Loads the second bitmap in the icon
Public Const ICO_THIRD           As Long = 0     ' [In    ] Loads the third bitmap in the icon
Public Const IFF_DEFAULT         As Long = 0     ' [In/Out] Default flag
Public Const JPEG_DEFAULT        As Long = 0     ' [In/Out] Default flag
Public Const JPEG_FAST           As Long = 1     ' [In    ] Loads the file as fast as possible, sacrificing some quality
Public Const JPEG_ACCURATE       As Long = 2     ' [In    ] Loads the file with the best quality, sacrificing some speed
Public Const JPEG_QUALITYSUPERB  As Long = &H80  ' [   Out] Saves with superb quality
Public Const JPEG_QUALITYGOOD    As Long = &H100 ' [   Out] Saves with good quality
Public Const JPEG_QUALITYNORMAL  As Long = &H200 ' [   Out] Saves with normal quality
Public Const JPEG_QUALITYAVERAGE As Long = &H400 ' [   Out] Saves with average quality
Public Const JPEG_QUALITYBAD     As Long = &H800 ' [   Out] Saves with bad quality
Public Const KOALA_DEFAULT       As Long = 0     ' [In/Out] Default flag
Public Const LBM_DEFAULT         As Long = 0     ' [In/Out] Default flag
Public Const MNG_DEFAULT         As Long = 0     ' [In/Out] Default flag
Public Const PCD_DEFAULT         As Long = 0     ' [In/Out] Default flag
Public Const PCD_BASE            As Long = 1     ' [In    ] A PhotoCD picture comes in many sizes. This flag will load the one sized 768 x 512
Public Const PCD_BASEDIV4        As Long = 2     ' [In    ] This flag will load the bitmap sized 384 x 256
Public Const PCD_BASEDIV16       As Long = 3     ' [In    ] This flag will load the bitmap sized 192 x 128
Public Const PCX_DEFAULT         As Long = 0     ' [In/Out] Default flag
Public Const PNG_DEFAULT         As Long = 0     ' [In/Out] Default flag
Public Const PNG_IGNOREGAMMA     As Long = 1     ' [In    ] Avoid gamma correction
Public Const PNM_DEFAULT         As Long = 0     ' [In/Out] Default flag
Public Const PNM_SAVE_RAW        As Long = 0     ' [   Out] If set the writer saves in RAW format (i.e. P4, P5 or P6)
Public Const PNM_SAVE_ASCII      As Long = 1     ' [   Out] If set the writer saves in ASCII format (i.e. P1, P2 or P3)
Public Const RAS_DEFAULT         As Long = 0     ' [In/Out] Default flag
Public Const TARGA_DEFAULT       As Long = 0     ' [In/Out] Default flag
Public Const TARGA_LOAD_RGB888   As Long = 1     ' [In    ] If set the loader converts RGB555 and ARGB8888 -> RGB888.
Public Const TARGA_LOAD_RGB555   As Long = 2     ' >> This flag is obsolete <<
Public Const TIFF_DEFAULT        As Long = 0     ' [In/Out] Default flag
Public Const WBMP_DEFAULT        As Long = 0     ' [In/Out] Default flag
Public Const PSD_DEFAULT         As Long = 0     ' [In/Out] Default flag

'_____________________________________________________________________________________________________________
' Other Constants
'
Public Const MAX_PATH     As Long = 260
Public Const BI_RGB       As Long = 0 ' An uncompressed format.
Public Const BI_RLE8      As Long = 1 ' A run-length encoded (RLE) format for bitmaps with 8 bits per pixel. The compression format is a two-byte format consisting of a count byte followed by a byte containing a color index. For more information, see the following Remarks section.
Public Const BI_RLE4      As Long = 2 ' An RLE format for bitmaps with 4 bits per pixel. The compression format is a two-byte format consisting of a count byte followed by two word-length color indices. For more information, see the following Remarks section.
Public Const BI_BITFIELDS As Long = 3 ' Specifies that the bitmap is not compressed and that the color table consists of three doubleword color masks that specify the red, green, and blue components, respectively, of each pixel. This is valid when used with 16- and 32-bits-per-pixel bitmaps.


'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXX                        SUB / FUNCTION  DECLARATIONS                     XXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


'_____________________________________________________________________________________________________________
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
' Init/Error routines
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'

'-------------------------------------------------------------------------------------------------------------
' Initialises the library.
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_Initialise(BOOL load_local_plugins_only FI_DEFAULT(FALSE));
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_Initialise Lib "FREEIMAGE.DLL" Alias "_FreeImage_Initialise@4" (Optional ByVal load_local_plugins_only As BOOL = FALSE_)

'-------------------------------------------------------------------------------------------------------------
' DeInitialises the library.
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_DeInitialise();
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_DeInitialise Lib "FREEIMAGE.DLL" Alias "_FreeImage_DeInitialise@0" ()

'_____________________________________________________________________________________________________________
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
' Version routines
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'

'-------------------------------------------------------------------------------------------------------------
' Returns the version string of the library.
'-------------------------------------------------------------------------------------------------------------
'DLL_API const char *DLL_CALLCONV FreeImage_GetVersion();
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetVersion Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetVersion@0" () As Long

'-------------------------------------------------------------------------------------------------------------
' Returns a short copyright message you can include in your program.
'-------------------------------------------------------------------------------------------------------------
'DLL_API const char *DLL_CALLCONV FreeImage_GetCopyrightMessage();
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetCopyrightMessage Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetCopyrightMessage@0" () As String 'Long

'_____________________________________________________________________________________________________________
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
' Message output functions
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'

'-------------------------------------------------------------------------------------------------------------
' Callback function definition
'-------------------------------------------------------------------------------------------------------------
'typedef void (*FreeImage_OutputMessageFunction)(FREE_IMAGE_FORMAT fif, const char *msg);
'-------------------------------------------------------------------------------------------------------------
' (See the public function "FI_OutputMessageProc")

'-------------------------------------------------------------------------------------------------------------
' Sets the Message Proc equal to the function you specify (in Visual Basic, use the "AddressOf" operator to
' get the memory address of a PUBLIC FUNCTION [which must be located in a standard VB module] to act as this
' proc)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_SetOutputMessage(FreeImage_OutputMessageFunction omf);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_SetOutputMessage Lib "FREEIMAGE.DLL" Alias "_FreeImage_SetOutputMessage@4" (ByVal Msg_Callback_Function As Long)

'_____________________________________________________________________________________________________________
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
' Allocate/Unload routines
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'

'-------------------------------------------------------------------------------------------------------------
' Allocates a new FreeImage bitmap using the given width, height and bits per pixel and optional red, green
' and blue mask. The mask is stored in BGR order.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_Allocate(int width, int height, int bpp, unsigned red_mask FI_DEFAULT(0), unsigned green_mask FI_DEFAULT(0), unsigned blue_mask FI_DEFAULT(0));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_Allocate Lib "FREEIMAGE.DLL" Alias "_FreeImage_Allocate@24" (ByVal Width As Long, ByVal Height As Long, ByVal BitsPerPixel As Long, Optional ByVal Red_Mask As Long = 0, Optional ByVal Green_Mask As Long = 0, Optional ByVal Blue_Mask As Long = 0) As Long

'-------------------------------------------------------------------------------------------------------------
' Disposes the given bitmap from memory.
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_Free(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_Free Lib "FREEIMAGE.DLL" Alias "_FreeImage_Free@4" (ByVal DIB As Long)

'-------------------------------------------------------------------------------------------------------------
' Alias for FreeImage_Free.
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_Unload(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_Unload Lib "FREEIMAGE.DLL" Alias "_FreeImage_Unload@4" (ByVal DIB As Long)

'-------------------------------------------------------------------------------------------------------------
' Makes an exact copy of an existing bitmap in memory
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP * DLL_CALLCONV FreeImage_Clone(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_Clone Lib "FREEIMAGE.DLL" Alias "_FreeImage_Clone@4" (ByVal DIB As Long) As Long

'_____________________________________________________________________________________________________________
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
' Plugin Interface
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'

'-------------------------------------------------------------------------------------------------------------
' Registers a plugin which is residing in the programmer's own sourcebase (not a DLL)
'-------------------------------------------------------------------------------------------------------------
'DLL_API FREE_IMAGE_FORMAT DLL_CALLCONV FreeImage_RegisterLocalPlugin(FI_InitProc proc_address, const char *format FI_DEFAULT(0), const char *description FI_DEFAULT(0), const char *extension FI_DEFAULT(0), const char *regexpr FI_DEFAULT(0));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_RegisterLocalPlugin Lib "FREEIMAGE.DLL" Alias "_FreeImage_RegisterLocalPlugin@20" (ByVal Proc_Address As Long, Optional ByVal Format As String = vbNullString, Optional ByVal Description As String = vbNullString, Optional ByVal Extention As String = vbNullString, Optional ByVal RegExpr As String = vbNullString) As FREE_IMAGE_FORMAT

'-------------------------------------------------------------------------------------------------------------
' Register a plugin residing in a DLL
'-------------------------------------------------------------------------------------------------------------
'DLL_API FREE_IMAGE_FORMAT DLL_CALLCONV FreeImage_RegisterExternalPlugin(const char *path, const char *format FI_DEFAULT(0), const char *description FI_DEFAULT(0), const char *extension FI_DEFAULT(0), const char *regexpr FI_DEFAULT(0));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_RegisterExternalPlugin Lib "FREEIMAGE.DLL" Alias "_FreeImage_RegisterExternalPlugin@20" (ByVal Path As String, Optional ByVal Format As String = vbNullString, Optional ByVal Description As String = vbNullString, Optional ByVal Extention As String = vbNullString, Optional ByVal RegExpr As String = vbNullString) As FREE_IMAGE_FORMAT

'-------------------------------------------------------------------------------------------------------------
' Returns the number of supported bitmap formats.
'-------------------------------------------------------------------------------------------------------------
'DLL_API int DLL_CALLCONV FreeImage_GetFIFCount();
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetFIFCount Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetFIFCount@0" () As Long

'-------------------------------------------------------------------------------------------------------------
' Disabled and enables a plugin. If you disable a plug-in (file format), is  will no longer be possible
' to load and save using that plug-in.
'-------------------------------------------------------------------------------------------------------------
'DLL_API int DLL_CALLCONV FreeImage_SetPluginEnabled(FREE_IMAGE_FORMAT fif, BOOL enable);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_SetPluginEnabled Lib "FREEIMAGE.DLL" Alias "_FreeImage_SetPluginEnabled@8" (ByVal ImageFormat As FREE_IMAGE_FORMAT, ByVal Enabled As BOOL) As Long

'-------------------------------------------------------------------------------------------------------------
' Returns the status of the plugin (Enabled / Disabled)
'-------------------------------------------------------------------------------------------------------------
'DLL_API int DLL_CALLCONV FreeImage_IsPluginEnabled(FREE_IMAGE_FORMAT fif);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_IsPluginEnabled Lib "FREEIMAGE.DLL" Alias "_FreeImage_IsPluginEnabled@4" (ByVal ImageFormat As FREE_IMAGE_FORMAT) As Long

'-------------------------------------------------------------------------------------------------------------
' Returns the FreeImage format ID for the given format string.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FREE_IMAGE_FORMAT DLL_CALLCONV FreeImage_GetFIFFromFormat(const char *format);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetFIFFromFormat Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetFIFFromFormat@4" (ByVal Format As String) As FREE_IMAGE_FORMAT

'-------------------------------------------------------------------------------------------------------------
' Returns a plugin constant (FIF_BMP, FIF_TIFF, etc.) from a mime string (file extention)
'-------------------------------------------------------------------------------------------------------------
'DLL_API FREE_IMAGE_FORMAT DLL_CALLCONV FreeImage_GetFIFFromMime(const char *mime);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetFIFFromMime Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetFIFFromMime@4" (ByVal Mime As String) As FREE_IMAGE_FORMAT

'-------------------------------------------------------------------------------------------------------------
' Returns a format string for the given FreeImage Format ID.
'-------------------------------------------------------------------------------------------------------------
'DLL_API const char *DLL_CALLCONV FreeImage_GetFormatFromFIF(FREE_IMAGE_FORMAT fif);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetFormatFromFIF Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetFormatFromFIF@4" (ByVal ImageFormat As FREE_IMAGE_FORMAT) As String

'-------------------------------------------------------------------------------------------------------------
' Returns a list of possible file extension for the given FreeImage Format ID. The extension list is a string
' containing one or more comma separated file extensions.
'-------------------------------------------------------------------------------------------------------------
'DLL_API const char *DLL_CALLCONV FreeImage_GetFIFExtensionList(FREE_IMAGE_FORMAT fif);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetFIFExtensionList Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetFIFExtensionList@4" (ByVal ImageFormat As FREE_IMAGE_FORMAT) As String

'-------------------------------------------------------------------------------------------------------------
' Returns a descriptive string for the given FreeImage Format ID.
'-------------------------------------------------------------------------------------------------------------
'DLL_API const char *DLL_CALLCONV FreeImage_GetFIFDescription(FREE_IMAGE_FORMAT fif);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetFIFDescription Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetFIFDescription@4" (ByVal ImageFormat As FREE_IMAGE_FORMAT) As String

'-------------------------------------------------------------------------------------------------------------
' Returns a regular expression for the given FreeImage Format ID, assisting external libraries to identify the
' bitmap format. Currently FreeImageQt uses this function.
'-------------------------------------------------------------------------------------------------------------
'DLL_API const char * DLL_CALLCONV FreeImage_GetFIFRegExpr(FREE_IMAGE_FORMAT fif);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetFIFRegExpr Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetFIFDescription@4" (ByVal ImageFormat As FREE_IMAGE_FORMAT) As String

'-------------------------------------------------------------------------------------------------------------
' Tries to identify a bitmap type by looking at the filename or a file extension. FreeImage_GetFIFFromFilename
' returns a valid FreeImage Format ID on success and FIF_UNKNOWN on failure.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FREE_IMAGE_FORMAT DLL_CALLCONV FreeImage_GetFIFFromFilename(const char *filename);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetFIFFromFilename Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetFIFFromFilename@4" (ByVal FileName As String) As FREE_IMAGE_FORMAT

'-------------------------------------------------------------------------------------------------------------
' Returns true if the plugin belonging to this FreeImage Format ID supports reading, false otherwise.
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_FIFSupportsReading(FREE_IMAGE_FORMAT fif);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_FIFSupportsReading Lib "FREEIMAGE.DLL" Alias "_FreeImage_FIFSupportsReading@4" (ByVal ImageFormat As FREE_IMAGE_FORMAT) As BOOL

'-------------------------------------------------------------------------------------------------------------
' Returns true if the plugin belonging to this FreeImage Format ID supports writing, false otherwise.
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_FIFSupportsWriting(FREE_IMAGE_FORMAT fif);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_FIFSupportsWriting Lib "FREEIMAGE.DLL" Alias "_FreeImage_FIFSupportsWriting@4" (ByVal ImageFormat As FREE_IMAGE_FORMAT) As BOOL

'-------------------------------------------------------------------------------------------------------------
' Returns TRUE if the plug-in can export a bitmap in the desired color depth (bits per pixel)
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_FIFSupportsExportBPP(FREE_IMAGE_FORMAT fif, int bpp);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_FIFSupportsExportBPP Lib "FREEIMAGE.DLL" Alias "_FreeImage_FIFSupportsExportBPP@8" (ByVal ImageFormat As FREE_IMAGE_FORMAT, ByVal BitsPerPixel As Long) As BOOL

'-------------------------------------------------------------------------------------------------------------
' Loads the bitmap file into a FreeImage bitmap. If the bitmap is loaded successfully, memory for it is
' allocated and a bitmap pointer is returned. If the bitmap couldn 't be loaded, FreeImage_Load returns NULL.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_Load(FREE_IMAGE_FORMAT fif, const char *filename, int flags FI_DEFAULT(0));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_Load Lib "FREEIMAGE.DLL" Alias "_FreeImage_Load@12" (ByVal ImageFormat As FREE_IMAGE_FORMAT, ByVal FileName As String, Optional ByVal Flags As Long = 0) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads a bitmap file into a FreeImage bitmap using the specified FreeImageIO struct and fi_handle. If the
' file is loaded successfully, memory for it is allocated and a bitmap pointer is returned. If the file
' couldn't be loaded, FreeImage_LoadFromHandle returns NULL.
'
' The fif parameter specifies the desired bitmap format to be loaded. The parameter accepts any valid
' FreeImage Format ID, such as FIF_BMP or FIF_JPEG, or an integer value obtained from any of the plugin
' interface functions.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadFromHandle(FREE_IMAGE_FORMAT fif, FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(0));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadFromHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadBMPFromHandle@12" (ByVal ImageFormat As FREE_IMAGE_FORMAT, ByRef IO_Callback_Functions As FreeImageIO, Optional ByVal Flags As Long = 0) As Long

'-------------------------------------------------------------------------------------------------------------
' Saves the FreeImage DIB to a bitmap file of the given type.
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_Save(FREE_IMAGE_FORMAT fif, FIBITMAP *dib, const char *filename, int flags FI_DEFAULT(0));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_Save Lib "FREEIMAGE.DLL" Alias "_FreeImage_Save@16" (ByVal ImageFormat As FREE_IMAGE_FORMAT, ByVal DIB As Long, ByVal FileName As String, Optional ByVal Flags As Long = 0) As BOOL

'-------------------------------------------------------------------------------------------------------------
' Saves the FreeImage DIB to a bitmap file of the given type.
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_SaveToHandle(FREE_IMAGE_FORMAT fif, FIBITMAP *dib, FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(0));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_SaveToHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_SaveToHandle@20" (ByVal ImageFormat As FREE_IMAGE_FORMAT, ByVal DIB As Long, ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = 0) As BOOL
                                                     
'_____________________________________________________________________________________________________________
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
' Multipaging interface
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'

'-------------------------------------------------------------------------------------------------------------
' When using the multi-paging system, things are a bit different what you are used to. The bitmaps you retrieve
' from a multi-bitmap (e.g. a TIFF with multiple pages) are 'managed', which means that they are loaded and
' unloaded by the multi-page system as it sees fit. Because of this you may never call
' FreeImage_Unload/FreeImage_Free on a bitmap retrieved using FreeImage_LockPage, but instead must call
' FreeImage_unlockPage.
'
' The parameters in FreeImage_OpenMultiBitmap mean:
' 1) Selector for the bitmap plugin. currently the only one supported is FIF_TIFF
' 2) Name of the file to be opened
' 3) True to create a new bitmap, false to open an existing one
' 4) True to open a bitmap read-only, false to allow writing. ignored when the previous parameter is true
' 5) True if the multi-page system should keep all gathered bitmap data in memory, false to flush lazy to disc
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIMULTIBITMAP * DLL_CALLCONV FreeImage_OpenMultiBitmap(FREE_IMAGE_FORMAT fif, const char *filename, BOOL create_new, BOOL read_only, BOOL keep_cache_in_memory FI_DEFAULT(FALSE));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_OpenMultiBitmap Lib "FREEIMAGE.DLL" Alias "_FreeImage_OpenMultiBitmap@20" (ByVal ImageFormat As FREE_IMAGE_FORMAT, ByVal FileName As String, ByVal CreateNew As BOOL, ByVal ReadOnly As BOOL, Optional ByVal KeepCacheInMemory As BOOL = FALSE_) As Long

'-------------------------------------------------------------------------------------------------------------
' Closes a MultiBitmap opened with a successfull call to the "FreeImage_OpenMultiBitmap" function
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_CloseMultiBitmap(FIMULTIBITMAP *bitmap);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_CloseMultiBitmap Lib "FREEIMAGE.DLL" Alias "_FreeImage_CloseMultiBitmap@4" (ByVal MultipageBitmap As Long) As BOOL

'-------------------------------------------------------------------------------------------------------------
' Gets the total number of pages contained within a MultiBitmap
'-------------------------------------------------------------------------------------------------------------
'DLL_API int DLL_CALLCONV FreeImage_GetPageCount(FIMULTIBITMAP *bitmap);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetPageCount Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetPageCount@4" (ByVal MultipageBitmap As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Appends a page to an opened MultiBitmap
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_AppendPage(FIMULTIBITMAP *bitmap, FIBITMAP *data);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_AppendPage Lib "FREEIMAGE.DLL" Alias "_FreeImage_AppendPage@8" (ByVal MultipageBitmap As Long, ByVal Data As Long)

'-------------------------------------------------------------------------------------------------------------
' Inserts a page into a MultiBitmap
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_InsertPage(FIMULTIBITMAP *bitmap, int page, FIBITMAP *data);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_InsertPage Lib "FREEIMAGE.DLL" Alias "_FreeImage_InsertPage@12" (ByVal MultipageBitmap As Long, ByVal PageNumber As Long, ByVal Data As Long)

'-------------------------------------------------------------------------------------------------------------
' Deletes a page from a MultiBitmap
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_DeletePage(FIMULTIBITMAP *bitmap, int page);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_DeletePage Lib "FREEIMAGE.DLL" Alias "_FreeImage_DeletePage@8" (ByVal MultipageBitmap As Long, ByVal PageNumber As Long)

'-------------------------------------------------------------------------------------------------------------
' Locks a specified page from a MultiBitmap so it can be saved or manipulated
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP * DLL_CALLCONV FreeImage_LockPage(FIMULTIBITMAP *bitmap, int page);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LockPage Lib "FREEIMAGE.DLL" Alias "_FreeImage_LockPage@8" (ByVal MultipageBitmap As Long, ByVal PageNumber As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Unlocks a specified page from a MultiBitmap
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_UnlockPage(FIMULTIBITMAP *bitmap, FIBITMAP *page, BOOL changed);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_UnlockPage Lib "FREEIMAGE.DLL" Alias "_FreeImage_UnlockPage@12" (ByVal MultipageBitmap As Long, ByVal Page As Long, ByVal Changed As BOOL)

'-------------------------------------------------------------------------------------------------------------
' Relocates the specified page in a MultiBitmap
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_MovePage(FIMULTIBITMAP *bitmap, int target, int source);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_MovePage Lib "FREEIMAGE.DLL" Alias "_FreeImage_MovePage@12" (ByVal MultipageBitmap As Long, ByVal Target As Long, ByVal Source As Long) As BOOL

'-------------------------------------------------------------------------------------------------------------
' Returns an array of page-numbers that are currently locked in memory.  When the pages parameter is NULL,
' in count the size of the array is returned.  You can then allocate the array of the desired size and call
' FreeImage_GetLockedPageNumbers again to populate the array.
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_GetLockedPageNumbers(FIMULTIBITMAP *bitmap, int *pages, int *count);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetLockedPageNumbers Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetLockedPageNumbers@12" (ByVal MultipageBitmap As Long, ByRef Pages() As Long, ByRef Count As Long) As BOOL

'_____________________________________________________________________________________________________________
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
' Old style bitmap load routines (deprecated)
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'

'-------------------------------------------------------------------------------------------------------------
' Loads the given Windows Bitmap file or OS/2 Bitmap file into a FreeImage bitmap. If the bitmap is loaded
' successfully, memory for it is allocated and a bitmap pointer is returned. If the bitmap couldn't be loaded,
' FreeImage_LoadBMP returns NULL.
'
' FIBITMAP *dib = FreeImage_LoadBMP("test.bmp");
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadBMP(const char *filename, int flags FI_DEFAULT(BMP_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadBMP Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadBMP@8" (ByVal FileName As String, Optional ByVal Flags As Long = BMP_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads a Dr.Halo file (deprecated)
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadCUT(const char *filename, int flags FI_DEFAULT(CUT_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadCUT Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadCUT@8" (ByVal FileName As String, Optional ByVal Flags As Long = CUT_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given Windows ICON file into a FreeImage bitmap. If the icon is loaded successfully, memory for
' it is allocated and a bitmap pointer is returned. If the bitmap couldn't be loaded, FreeImage_LoadICO
' returns NULL.
'
' Flags defaults to ICO_DEFAULT. The following parameters can be passed to change the behaviour of the load
' routine:
'
' FIBITMAP *dib = FreeImage_LoadICO("test.ico");
' FIBITMAP *dib = FreeImage_LoadICO("test.ico", ICO_SECOND);
'
' Flag Value:               Meaning:
' ICO_FIRST            Loads the first icon in the cabinet
' ICO_SECOND           Loads the second icon in the cabinet
' ICO_THIRD            Loads the third icon in the cabinet
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadICO(const char *filename, int flags FI_DEFAULT(ICO_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadICO Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadICO@8" (ByVal FileName As String, Optional ByVal Flags As Long = ICO_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads a Interchanged File Format file (deprecated)
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadIFF(const char *filename, int flags FI_DEFAULT(IFF_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadIFF Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadIFF@8" (ByVal FileName As String, Optional ByVal Flags As Long = IFF_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given JPEG or JIF file into a FreeImage bitmap. If the file is loaded successfully, memory for it
' is allocated and a bitmap pointer is returned. If the file couldn't be loaded, FreeImage_LoadJPEG returns
' NULL.

' Flags defaults to JPEG_DEFAULT. The following parameters can be passed to the flags parameter to change
' the behaviour of the load routine:
'
' FIBITMAP *dib = FreeImage_LoadJPEG("test.jpg");
' FIBITMAP *dib = FreeImage_LoadJPEG("test.jpg", JPEG_ACCURATE);
'
' Flag Value:               Meaning:
' JPEG_FAST            Loads the file as fast as possible, sacrificing some quality.
' JPEG_ACCURATE        Loads the file as accurate as possible, sacrificing some speed.
' JPEG_FAST            (undocumented)
' JPEG_ACCURATE        (undocumented)
' JPEG_QUALITYSUPERB   (undocumented)
' JPEG_QUALITYGOOD     (undocumented)
' JPEG_QUALITYNORMAL   (undocumented)
' JPEG_QUALITYAVERAGE  (undocumented)
' JPEG_QUALITYBAD      (undocumented)
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadJPEG(const char *filename, int flags FI_DEFAULT(JPEG_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadJPEG Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadJPEG@8" (ByVal FileName As String, Optional ByVal Flags As Long = JPEG_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given KOALA file into a FreeImage bitmap. If the file is loaded successfully, memory for it is
' allocated and a bitmap pointer is returned. If the file couldn 't be loaded, FreeImage_LoadKOALA returns NULL
'
' KOALA is a 4-bit bitmap format developed for and supported by ancient commodore 64 computers and emulators.
' The format is supported for nostalgic reasons of the author.
'
' FIBITMAP *dib = FreeImage_LoadKOALA("test.koa");
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadKOALA(const char *filename, int flags FI_DEFAULT(KOALA_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadKOALA Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadKOALA@8" (ByVal FileName As String, Optional ByVal Flags As Long = KOALA_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given Deluxe Paint LBM file into a FreeImage bitmap. If the file is loaded successfully, memory
' for it is allocated and a bitmap pointer is returned. If the file couldn't be loaded, FreeImage_LoadLBM
' returns NULL
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadLBM(const char *filename, int flags FI_DEFAULT(LBM_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadLBM Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadLBM@8" (ByVal FileName As String, Optional ByVal Flags As Long = LBM_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given Multi Network Graphics (MNG) or JPEG Network Graphics (JNG) file into a FreeImage bitmap.
' If the file is loaded successfully, memory for it is allocated and a bitmap pointer is returned. If the file
' couldn't be loaded, FreeImage_LoadMNG returns NULL
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadMNG(const char *filename, int flags FI_DEFAULT(MNG_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadMNG Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadMNG@8" (ByVal FileName As String, Optional ByVal Flags As Long = MNG_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given Kodak PhotoCD file into a FreeImage bitmap. If the file is loaded successfully, memory for
' it is allocated and a bitmap pointer is returned. If the file couldn't be loaded, FreeImage_LoadPCD returns
' NULL.
'
' PhotoCD images are so called cabinet files, where several versions of the same bitmap exist in one file.
' By default FreeImage loads the image marked in the flags as PCD_BASE. By using the flags parameter other
' image versions can be obtained:
'
' FIBITMAP *dib = FreeImage_LoadPCD("test.pcd");
' FIBITMAP *dib = FreeImage_LoadPCX("test.pcd", PCD_BASEDIV16);
'
' Flag Value:               Meaning:
' PCD_BASE             24-bit, 768 x 512 pixels
' PCD_BASEDIV4         24-bit, 192 x 128 pixels
' PCD_BASEDIV16        24-bit. 384 x 256 pixels
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadPCD(const char *filename, int flags FI_DEFAULT(PCD_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadPCD Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadPCD@8" (ByVal FileName As String, Optional ByVal Flags As Long = PCD_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given PCX file into a FreeImage bitmap. If the file is loaded successfully, memory for it is
' allocated and a bitmap pointer is returned. If the file couldn't be loaded, FreeImage_LoadPCX returns NULL.
'
' FIBITMAP *dib = FreeImage_LoadPCX("test.pcx");
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadPCX(const char *filename, int flags FI_DEFAULT(PCX_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadPCX Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadPCX@8" (ByVal FileName As String, Optional ByVal Flags As Long = PCX_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given PNG (Portable Network Graphic) file into a FreeImage bitmap. If the file is loaded
' successfully, memory for it is allocated and a bitmap pointer is returned. If the file couldn't be loaded,
' FreeImage_LoadPNG returns NULL.
'
' FIBITMAP *dib = FreeImage_LoadPNG("test.png");
'
' Flag Value:               Meaning:
' PNG_IGNOREGAMMA      Avoid gamma correction
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadPNG(const char *filename, int flags FI_DEFAULT(PNG_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadPNG Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadPNG@8" (ByVal FileName As String, Optional ByVal Flags As Long = PNG_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads one of the PNM supported bitmap files into a FreeImage bitmap. If the file is loaded successfully,
' memory for it is allocated and a bitmap pointer is returned. If the file couldn't be loaded,
' FreeImage_LoadPNM returns NULL.
'
' PNM is a descriptive name for a collection of ASCII based bitmap types: PBM, PGM and PPM. PBM (Portable
' Bitmap format) is a 1-bit, black and white bitmap type, PGM (Portable Greymap format) is an 8-bit
' greyscale bitmap type and PPM (Portable Pixelmap format) is a 24-bit full colour bitmap.  FreeImage
' automatically detects if you are dealing with a PBM, PGM or PPM bitmap and will load it in the default bit
' depth of that particular image.
'
' FIBITMAP *dib = FreeImage_LoadPNM("test.ppm");
' FIBITMAP *dib = FreeImage_LoadPNM("test.pgm");
' FIBITMAP *dib = FreeImage_LoadPNM("test.pbm");
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadPNM(const char *filename, int flags FI_DEFAULT(PNM_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadPNM Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadPNM@8" (ByVal FileName As String, Optional ByVal Flags As Long = PNM_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads a photoshop file (deprecated)
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadPSD(const char *filename, int flags FI_DEFAULT(PSD_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadPSD Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadPSD@8" (ByVal FileName As String, Optional ByVal Flags As Long = PSD_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given Sun rasterfile into a FreeImage bitmap. If the file is loaded successfully, memory for it
' is allocated and a bitmap pointer is returned. If the file couldn 't be loaded, FreeImage_LoadRAS returns
' NULL.
'
' FIBITMAP *dib = FreeImage_LoadRAS("test.ras");
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadRAS(const char *filename, int flags FI_DEFAULT(RAS_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadRAS Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadRAS@8" (ByVal FileName As String, Optional ByVal Flags As Long = RAS_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given TARGA file into a FreeImage bitmap. If the file is loaded successfully, memory for it is
' allocated and a bitmap pointer is returned. If the file couldn 't be loaded, FreeImage_LoadTARGA returns NULL.
'
' Flags defaults to TARGA_DEFAULT. The flags parameter can be passed the following values to change the
' behaviour of the load plugin:
'
' FIBITMAP *dib = FreeImage_LoadTARGA("test.tga");
' FIBITMAP *dib = FreeImage_LoadTARGA("test.tga", TARGA_LOAD_RGB888);
'
' Flag Value:               Meaning:
' TARGA_LOAD_RGB888    If the TARGA file is 16 or 32-bit, it is automatically converted to 24-bit.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadTARGA(const char *filename, int flags FI_DEFAULT(TARGA_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadTARGA Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadTARGA@8" (ByVal FileName As String, Optional ByVal Flags As Long = TARGA_LOAD_RGB888) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given TIFF file into a FreeImage bitmap. If the file is loaded successfully, memory for it is
' allocated and a bitmap pointer is returned. If the file couldn't be loaded, FreeImage_LoadTIFF returns NULL.
'
' FIBITMAP *dib = FreeImage_LoadTIFF("test.tiff");
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadTIFF(const char *filename, int flags FI_DEFAULT(TIFF_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadTIFF Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadTIFF@8" (ByVal FileName As String, Optional ByVal Flags As Long = TIFF_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given WBMP or WAP file into a FreeImage bitmap. If the file is loaded successfully, memory for it
' is allocated and a bitmap pointer is returned. If the file couldn't be loaded, FreeImage_LoadWBMP returns
' NULL.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadWBMP(const char *filename, int flags FI_DEFAULT(WBMP_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadWBMP Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadWBMP@8" (ByVal FileName As String, Optional ByVal Flags As Long = WBMP_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given BMP file into a FreeImage bitmap using the specified FreeImageIO struct and fi_handle. If
' the file is loaded successfully, memory for it is allocated and a bitmap pointer is returned. If the file
' couldn't be loaded, FreeImage_LoadBMPFromHandle returns NULL.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadBMPFromHandle(FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(BMP_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadBMPFromHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadBMPFromHandle@12" (ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = BMP_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads a Dr.Halo file from a handle (deprecated)
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadCUTFromHandle(FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(CUT_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadCUTFromHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadCUTFromHandle@12" (ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = CUT_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given ICO file into a FreeImage bitmap using the given FreeImageIO struct and fi_handle. If the
' file is loaded successfully, memory for it is allocated and a bitmap pointer is returned. If the file
' couldn't be loaded, FreeImage_LoadICOFromHandle returns NULL.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadICOFromHandle(FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(ICO_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadICOFromHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadICOFromHandle@12" (ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = ICO_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given TIFF file into a FreeImage bitmap using the specified FreeImageIO struct and fi_handle. If
' the file is loaded successfully, memory for it is allocated and a bitmap pointer is returned. If the file
' couldn't be loaded, FreeImage_LoadTIFFFromHandle returns NULL.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadIFFFromHandle(FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(IFF_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadIFFFromHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadTIFFFromHandle@12" (ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = TIFF_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given JPEG file into a FreeImage bitmap using the specified FreeImageIO struct and fi_handle. If
' the file is loaded successfully, memory for it is allocated and a bitmap pointer is returned. If the file
' couldn't be loaded, FreeImage_LoadJPEGFromHandle returns NULL.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadJPEGFromHandle(FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(JPEG_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadJPEGFromHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadJPEGFromHandle@12" (ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = JPEG_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given KOALA file into a FreeImage bitmap using the specified FreeImageIO struct and fi_handle. If
' the file is loaded successfully, memory for it is allocated and a bitmap pointer is returned. If the file
' couldn't be loaded, FreeImage_LoadKOALAFromHandle returns NULL.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadKOALAFromHandle(FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(KOALA_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadKOALAFromHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadKOALAFromHandle@12" (ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = KOALA_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given Deluxe Paint LBM file into a FreeImage bitmap using the specified FreeImageIO struct and
' fi_handle. If the file is loaded successfully, memory for it is allocated and a bitmap pointer is returned.
' If the file couldn't be loaded, FreeImage_LoadLBMFromHandle returns NULL.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadLBMFromHandle(FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(LBM_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadLBMFromHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadLBMFromHandle@12" (ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = LBM_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the Multi Network Graphics (MNG) or JPEG Network Graphics (JNG) file into a FreeImage bitmap using
' the specified FreeImageIO struct and fi_handle. If the file is loaded successfully, memory for it is
' allocated and a bitmap pointer is returned. If the file couldn't be loaded, FreeImage_LoadMNGFromHandle
' returns NULL.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadMNGFromHandle(FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(MNG_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadMNGFromHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadMNGFromHandle@12" (ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = MNG_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given Kodak PhotoCD file into a FreeImage bitmap using the specified FreeImageIO struct and
' fi_handle. If the file is loaded successfully, memory for it is allocated and a bitmap pointer is returned.
' If the file couldn't be loaded, FreeImage_LoadPCDFromHandle returns NULL.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadPCDFromHandle(FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(PCD_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadPCDFromHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadPCDFromHandle@12" (ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = PCD_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given PCX file into a FreeImage bitmap using the specified FreeImageIO struct and fi_handle. If
' the file is loaded successfully, memory for it is allocated and a bitmap pointer is returned. If the file
' couldn't be loaded, FreeImage_LoadPCXFromHandle returns NULL.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadPCXFromHandle(FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(PCX_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadPCXFromHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadPCXFromHandle@12" (ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = PCX_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given PNG file into a FreeImage bitmap using the specified FreeImageIO struct and fi_handle. If
' the file is loaded successfully, memory for it is allocated and a bitmap pointer is returned. If the file
' couldn't be loaded, FreeImage_LoadPNGFromHandle returns NULL.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadPNGFromHandle(FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(PNG_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadPNGFromHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadPNGFromHandle@12" (ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = PNG_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given PBM, PGM or PPM file into a FreeImage bitmap using the specified FreeImageIO struct and
' fi_handle. If the file is loaded successfully, memory for it is allocated and a bitmap pointer is returned.
' If the file couldn't be loaded, FreeImage_LoadPNMFromHandle returns NULL.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadPNMFromHandle(FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(PNM_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadPNMFromHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadPNMFromHandle@12" (ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = PNM_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads a Photoshop Document from a handle (deprecated)
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadPSDFromHandle(FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(PSD_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadPSDFromHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadPSDFromHandle@12" (ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = PSD_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given Sun Rasterfile into a FreeImage bitmap using the specified FreeImageIO struct and fi_handle.
' If the file is loaded successfully, memory for it is allocated and a bitmap pointer is returned. If the file
' couldn't be loaded, FreeImage_LoadRASFromHandle returns NULL.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadRASFromHandle(FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(RAS_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadRASFromHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadRASFromHandle@12" (ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = RAS_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given TARGA file into a FreeImage bitmap using the specified FreeImageIO struct and fi_handle. If
' the file is loaded successfully, memory for it is allocated and a bitmap pointer is returned. If the file
' couldn't be loaded, FreeImage_LoadTARGAFromHandle returns NULL.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadTARGAFromHandle(FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(TARGA_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadTARGAFromHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadTARGAFromHandle@12" (ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = TARGA_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given TIFF file into a FreeImage bitmap using the specified FreeImageIO struct and fi_handle. If
' the file is loaded successfully, memory for it is allocated and a bitmap pointer is returned. If the file
' couldn't be loaded, FreeImage_LoadTIFFFromHandle returns NULL.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadTIFFFromHandle(FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(TIFF_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadTIFFFromHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadTIFFFromHandle@12" (ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = TIFF_DEFAULT) As Long

'-------------------------------------------------------------------------------------------------------------
' Loads the given WBMP file into a FreeImage bitmap using the specified FreeImageIO struct and fi_handle. If
' the file is loaded successfully, memory for it is allocated and a bitmap pointer is returned. If the file
' couldn't be loaded, FreeImage_LoadWBMPFromHandle returns NULL.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_LoadWBMPFromHandle(FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(WBMP_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_LoadWBMPFromHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_LoadWBMPFromHandle@12" (ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = WBMP_DEFAULT) As Long

'_____________________________________________________________________________________________________________
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
' Bitmap save routines
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'

'-------------------------------------------------------------------------------------------------------------
' Saves the FreeImage DIB to a Windows Bitmap file. The BMP file is always saved in the Windows format.
' No compression is used.
'
' FIBITMAP *dib = FreeImage_LoadBMP("test.bmp");
' if (dib != NULL) {
'   FreeImage_SaveBMP("saved.bmp");
'   FreeImage_Free(dib);
' }
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_SaveBMP(FIBITMAP *dib, const char *filename, int flags FI_DEFAULT(BMP_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_SaveBMP Lib "FREEIMAGE.DLL" Alias "_FreeImage_SaveBMP@12" (ByVal DIB As Long, ByVal FileName As String, Optional ByVal Flags As Long = BMP_DEFAULT) As BOOL

'-------------------------------------------------------------------------------------------------------------
' Saves the FreeImage DIB to a JPEG file. . Only 24-bit bitmaps can be saved as JPEG. Bitmaps in bit depths will have to be converted.
'
' FIBITMAP *dib = FreeImage_LoadBMP("test.bmp");
' if (dib != NULL) {
'   FreeImage_SaveJPEG("saved.jpg");
'   FreeImage_Free(dib);
' }
'
' Flag Value:               Meaning:
' JPEG_FAST
' JPEG_ACCURATE
' JPEG_QUALITYSUPERB
' JPEG_QUALITYGOOD
' JPEG_QUALITYNORMAL
' JPEG_QUALITYAVERAGE
' JPEG_QUALITYBAD
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_SaveJPEG(FIBITMAP *dib, const char *filename, int flags FI_DEFAULT(JPEG_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_SaveJPEG Lib "FREEIMAGE.DLL" Alias "_FreeImage_SaveJPEG@12" (ByVal DIB As Long, ByVal FileName As String, Optional ByVal Flags As Long = JPEG_DEFAULT) As BOOL

'-------------------------------------------------------------------------------------------------------------
' Saves the FreeImage DIB to a PNG file.
'
' FIBITMAP *dib = FreeImage_LoadBMP("test.bmp");
' if (dib != NULL) {
'   FreeImage_SavePNG("saved.png");
'   FreeImage_Free(dib);
' }
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_SavePNG(FIBITMAP *dib, const char *filename, int flags FI_DEFAULT( PNG_DEFAULT ));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_SavePNG Lib "FREEIMAGE.DLL" Alias "_FreeImage_SavePNG@12" (ByVal DIB As Long, ByVal FileName As String, Optional ByVal Flags As Long = PNG_DEFAULT) As BOOL

'-------------------------------------------------------------------------------------------------------------
' Saves the FreeImage DIB to a PNM file. PNM is a descriptive name for a collection of ASCII based bitmap
' types: PBM, PGM and PPM. If the bitmap has a bit depth of 1, the file is saved as a PBM file. If the
' bitmap has a bit depth of 8, the file is saved as a PGM file. If the bitmap has a bit depth of 24, the
' file is saved as a PPM file. Other bit depths are not supported.
'
' FIBITMAP *dib = FreeImage_LoadBMP("test.bmp");
'
' if (dib != NULL) {
'   switch(FreeImage_GetBPP(dib)) {
'     Case 1:
'       FreeImage_SavePNM("saved.pbm");
'       break;
'     Case 8:
'       FreeImage_SavePNM("saved.pgm");
'       break;
'     Case 24:
'       FreeImage_SavePNM("saved.ppm");
'       break;
'   }
'   FreeImage_Free(dib);
' }
'
' Flag Value:               Meaning:
' PNM_SAVE_RAW         If set the writer saves in RAW format (i.e. P4, P5 or P6)
' PNM_SAVE_ASCII       If set the writer saves in ASCII format (i.e. P1, P2 or P3)
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_SavePNM(FIBITMAP *dib, const char *filename, int flags FI_DEFAULT(PNM_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_SavePNM Lib "FREEIMAGE.DLL" Alias "_FreeImage_SavePNN@12" (ByVal DIB As Long, ByVal FileName As String, Optional ByVal Flags As Long = PNM_DEFAULT) As BOOL

'-------------------------------------------------------------------------------------------------------------
' Saves the FreeImage DIB to a TIFF file.
'
' FIBITMAP *dib = FreeImage_LoadBMP("test.bmp");
' if (dib != NULL) {
'   FreeImage_SaveTIFF("saved.tiff");
'   FreeImage_Free(dib);
' }
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_SaveTIFF(FIBITMAP *dib, const char *filename, int flags FI_DEFAULT(TIFF_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_SaveTIFF Lib "FREEIMAGE.DLL" Alias "_FreeImage_SaveTIFF@12" (ByVal DIB As Long, ByVal FileName As String, Optional ByVal Flags As Long = TIFF_DEFAULT) As BOOL

'-------------------------------------------------------------------------------------------------------------
' Saves the FreeImage DIB to a WBMP file.
'
' FIBITMAP *dib = FreeImage_LoadBMP("test.bmp");
' if (dib != NULL) {
'   FreeImage_SaveWBMP("saved.wbmp");
'   FreeImage_Free(dib);
' }
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_SaveWBMP(FIBITMAP *dib, const char *filename, int flags FI_DEFAULT(WBMP_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_SaveWBMP Lib "FREEIMAGE.DLL" Alias "_FreeImage_SaveWBMP@12" (ByVal DIB As Long, ByVal FileName As String, Optional ByVal Flags As Long = WBMP_DEFAULT) As BOOL

'-------------------------------------------------------------------------------------------------------------
' Saves the FreeImage DIB to a Windows Bitmap file. The BMP file is always saved in the Windows format.
' No compression is used.
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_SaveBMPToHandle(FIBITMAP *dib, FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(BMP_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_SaveBMPToHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_SaveBMPToHandle@16" (ByVal DIB As Long, ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = BMP_DEFAULT) As BOOL

'-------------------------------------------------------------------------------------------------------------
' Saves the FreeImage DIB to a JPEG file. Only 24-bit bitmaps can be saved as JPEG. Bitmaps in bit depths will
' have to be converted.
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_SaveJPEGToHandle(FIBITMAP *dib, FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(JPEG_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_SaveJPEGToHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_SaveJPEGToHandle@16" (ByVal DIB As Long, ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = JPEG_DEFAULT) As BOOL

'-------------------------------------------------------------------------------------------------------------
' Saves the FreeImage DIB to a PNG file.
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_SavePNGToHandle(FIBITMAP *dib, FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(PNG_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_SavePNGToHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_SavePNGToHandle@16" (ByVal DIB As Long, ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = PNG_DEFAULT) As BOOL

'-------------------------------------------------------------------------------------------------------------
' Saves the FreeImage DIB to a PNM file. PNM is a descriptive name for a collection of ASCII based bitmap
' types: PBM, PGM and PPM. If the bitmap has a bitdepth of 1, the file is saved as a PBM file. If the bitmap
' has a bitdepth of 8, the file is saved as a PGM file. If the bitmap has a bitdepth of 24, the file is saved
' as a PPM file. Other bitdepths are not supported.
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_SavePNMToHandle(FIBITMAP *dib, FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(PNM_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_SavePNMToHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_SavePNMToHandle@16" (ByVal DIB As Long, ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = PNM_DEFAULT) As BOOL

'-------------------------------------------------------------------------------------------------------------
' Saves the FreeImage DIB to a TIFF file.
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_SaveTIFFToHandle(FIBITMAP *dib, FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(TIFF_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_SaveTIFFToHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_SaveTIFFToHandle@16" (ByVal DIB As Long, ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = TIFF_DEFAULT) As BOOL

'-------------------------------------------------------------------------------------------------------------
' Saves the FreeImage DIB to a WBMP file.
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_SaveWBMPToHandle(FIBITMAP *dib, FreeImageIO *io, fi_handle handle, int flags FI_DEFAULT(WBMP_DEFAULT));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_SaveWBMPToHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_SaveWBMPToHandle@16" (ByVal DIB As Long, ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, Optional ByVal Flags As Long = WBMP_DEFAULT) As BOOL

'_____________________________________________________________________________________________________________
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
' Filetype request routines
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'

'-------------------------------------------------------------------------------------------------------------
' Investigates the bitmap data from the given bitmap and returns the FreeImage Format ID for it.
' FreeImage_GetFileType will read the first size bytes of the file but not more than 16. FreeImage_GetFileType
' will return one of the following values:
'
' Type          Bitmap
' FIF_UNKNOWN   Unidentified bitmap type
' FIF_BMP       Windows or OS/2 Bitmap
' FIF_ICO       Windows Icon
' FIF_JPEG      JPEG - JFIF Compliant
' FIF_JNG       JPEG Network Graphics
' FIF_KOALA     C64 Koala Graphics
' FIF_LBM       Deluxe Paint LBM
' FIF_MNG       Multiple Network Graphics
' FIF_PBM       Portable Bitmap
' FIF_PCD       Kodak PhotoCD
' FIF_PCX       Zsoft Paintbrush
' FIF_PGM       Portable Greymap
' FIF_PNG       Portable Network Graphics
' FIF_PPM       Portable Pixmap
' FIF_RAS       Sun Raster Image
' FIF_TARGA     Truevision Targa
' FIF_TIFF      Tagged Image File Format
' FIF_WBMP      Wireless Bitmap
' FIF_PSD       Photoshop Document
' FIF_CUT       ...
' FIF_IFF       (see LBM)
'
'-------------------------------------------------------------------------------------------------------------
'DLL_API FREE_IMAGE_FORMAT DLL_CALLCONV FreeImage_GetFileType(const char *filename, int size);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetFileType Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetFileType@8" (ByVal FileName As String, ByVal Size As Long) As FREE_IMAGE_FORMAT

'-------------------------------------------------------------------------------------------------------------
' Investigates the bitmap data from the given bitmap and returns the FreeImage Format ID for it.
'
' Code example:
'
' unsigned
' _ReadProc(FIBITMAP *buffer, unsigned s, unsigned c, fi_handle handle) {
'   return fread(buffer, s, c, (FILE *)handle);
' }
' unsigned
' _WriteProc(FIBITMAP *buffer, unsigned s, unsigned c, fi_handle handle){
'   return fwrite(buffer, s, c, (FILE *)handle);
' }
' int
' _SeekProc(fi_handle handle, long offset, int origin) {
'   return fseek((FILE *)handle, offset, origin);
' }
' long
' _TellProc(fi_handle handle) {
'   return ftell((FILE *)handle);
' }
' FreeImageIO io;
' io.read_proc  = _ReadProc;
' io.write_proc = _WriteProc;
' io.seek_proc  = _SeekProc;
' io.tell_proc  = _TellProc;
' FILE *file = fopen("test.bmp", "rb");
' FREE_IMAGE_TYPE type;
' type = FreeImage_GetFileTypeFromHandle(&io, (fi_handle)file, 16);
'
'-------------------------------------------------------------------------------------------------------------
'DLL_API FREE_IMAGE_FORMAT DLL_CALLCONV FreeImage_GetFileTypeFromHandle(FreeImageIO *io, fi_handle handle, int size);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetFileTypeFromHandle Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetFileTypeFromHandle@12" (ByRef IO_Callback_Functions As FreeImageIO, ByVal Handle As Long, ByVal Size As Long) As FREE_IMAGE_FORMAT

'-------------------------------------------------------------------------------------------------------------
' Takes the specified FreeImage Format ID and returns back the string representation of it.
'-------------------------------------------------------------------------------------------------------------
'DLL_API const char * DLL_CALLCONV FreeImage_GetFileTypeFromFormat(FREE_IMAGE_FORMAT fif);  // this function is deprecated
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetFileTypeFromFormat Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetFileTypeFromFormat@4" (ByVal ImageFormat As FREE_IMAGE_FORMAT) As String

'-------------------------------------------------------------------------------------------------------------
' This function is deprecated. FreeImage_GetFIFFromFilename has replaced its functionality.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FREE_IMAGE_FORMAT DLL_CALLCONV FreeImage_GetFileTypeFromExt(const char *filename); // this function is deprecated
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetFileTypeFromExt Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetFileTypeFromExt@4" (ByVal FileName As String) As FREE_IMAGE_FORMAT

'_____________________________________________________________________________________________________________
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
' FreeImage info routines
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'

'-------------------------------------------------------------------------------------------------------------
' Returns the red bitmask for the bitmap. If the bitmap is palletised 0 is returned.
'-------------------------------------------------------------------------------------------------------------
'DLL_API unsigned DLL_CALLCONV FreeImage_GetRedMask(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetRedMask Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetRedMask@4" (ByVal DIB As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Returns the green bitmask for the bitmap. If the bitmap is palletised 0 is returned.
'-------------------------------------------------------------------------------------------------------------
'DLL_API unsigned DLL_CALLCONV FreeImage_GetGreenMask(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetGreenMask Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetGreenMask@4" (ByVal DIB As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Returns the blue bitmask for the bitmap. If the bitmap is palletised 0 is returned.
'-------------------------------------------------------------------------------------------------------------
'DLL_API unsigned DLL_CALLCONV FreeImage_GetBlueMask(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetBlueMask Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetBlueMask@4" (ByVal DIB As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Returns the number of transparent colours stored into the transparency colour table of the given bitmap.
' Every palletised bitmap includes a transparency table containing up to 256 alpha values.
'-------------------------------------------------------------------------------------------------------------
'DLL_API unsigned DLL_CALLCONV FreeImage_GetTransparencyCount(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetTransparencyCount Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetTransparencyCount@4" (ByVal DIB As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Returns the transparency table assigned to this bitmap. If the bitmap doesn't contain a transparency table
' NULL is returned.
'-------------------------------------------------------------------------------------------------------------
'DLL_API BYTE * DLL_CALLCONV FreeImage_GetTransparencyTable(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetTransparencyTable Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetTransparencyTable@4" (ByVal DIB As Long) As Long 'RETURNS BYTE ARRAY... THIS DECLARATION WILL RETURN A POINTER TO THAT ARRAY

'-------------------------------------------------------------------------------------------------------------
' Flags a bitmap as having transparent areas.  Only used in palettized bitmaps.
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_SetTransparent(FIBITMAP *dib, BOOL enabled);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_SetTransparent Lib "FREEIMAGE.DLL" Alias "_FreeImage_SetTransparent@8" (ByVal DIB As Long, ByVal Enabled As BOOL)

'-------------------------------------------------------------------------------------------------------------
' Assigns a new transparency table to the bitmap. A transparency table consists of up to 256 bytes in an
' array, where a value of 0xFF in that table stands for completely opaque and 0x00 stands for completely
' invisible. The transparency table is used in TrollTech Qt to draw transparent bitmaps on a widget.
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_SetTransparencyTable(FIBITMAP *dib, BYTE *table, int count);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_SetTransparencyTable Lib "FREEIMAGE.DLL" Alias "_FreeImage_SetTransparencyTable@12" (ByVal DIB As Long, ByRef Table() As Byte, ByVal Count As Long)

'-------------------------------------------------------------------------------------------------------------
' Returns the transparency status flag.  Only used in palettized bitmaps.
'-------------------------------------------------------------------------------------------------------------
'DLL_API BOOL DLL_CALLCONV FreeImage_IsTransparent(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_IsTransparent Lib "FREEIMAGE.DLL" Alias "_FreeImage_IsTransparent@4" (ByVal DIB As Long) As BOOL

'_____________________________________________________________________________________________________________
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
' DIB info routines
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'

'-------------------------------------------------------------------------------------------------------------
' Retrieves the number of colours used in the bitmap. If the bitmap is non-palletised, 0 is returned.
'-------------------------------------------------------------------------------------------------------------
'DLL_API unsigned DLL_CALLCONV FreeImage_GetColorsUsed(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetColorsUsed Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetColorsUsed@4" (ByVal DIB As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Returns a pointer to the bitmap bits.
'-------------------------------------------------------------------------------------------------------------
'DLL_API BYTE *DLL_CALLCONV FreeImage_GetBits(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetBits Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetBits@4" (ByVal DIB As Long) As Long 'RETURNS BYTE ARRAY... THIS DECLARATION WILL RETURN A POINTER TO THAT ARRAY

'-------------------------------------------------------------------------------------------------------------
' Returns a pointer to the bitmap bits on the given column and row.
'-------------------------------------------------------------------------------------------------------------
'DLL_API BYTE *DLL_CALLCONV FreeImage_GetBitsRowCol(FIBITMAP *dib, int col, int row);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetBitsRowCol Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetBitsRowCol@12" (ByVal DIB As Long, ByVal Col As Long, ByVal Row As Long) As Long  'RETURNS BYTE ARRAY... THIS DECLARATION WILL RETURN A POINTER TO THAT ARRAY

'-------------------------------------------------------------------------------------------------------------
' Returns a pointer to the beginning of the bits of the given scanline.
'-------------------------------------------------------------------------------------------------------------
'DLL_API BYTE *DLL_CALLCONV FreeImage_GetScanLine(FIBITMAP *dib, int scanline);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetScanLine Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetScanLine@8" (ByVal DIB As Long, ByVal ScanLine As Long) As Long  'RETURNS BYTE ARRAY... THIS DECLARATION WILL RETURN A POINTER TO THAT ARRAY

'-------------------------------------------------------------------------------------------------------------
' Returns the bitdepth of the bitmap.
'-------------------------------------------------------------------------------------------------------------
'DLL_API unsigned DLL_CALLCONV FreeImage_GetBPP(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetBPP Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetBPP@4" (ByVal DIB As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Returns the width of the bitmap in pixels.
'-------------------------------------------------------------------------------------------------------------
'DLL_API unsigned DLL_CALLCONV FreeImage_GetWidth(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetWidth Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetWidth@4" (ByVal DIB As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Returns the height of the bitmap in pixels.
'-------------------------------------------------------------------------------------------------------------
'DLL_API unsigned DLL_CALLCONV FreeImage_GetHeight(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetHeight Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetHeight@4" (ByVal DIB As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Returns the width of a bitmap in BYTES (not pixels)
'-------------------------------------------------------------------------------------------------------------
'DLL_API unsigned DLL_CALLCONV FreeImage_GetLine(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetLine Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetLine@4" (ByVal DIB As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Returns the width of the bitmap in BYTES (instead of pixels) rounded to the nearest DWORD.
'-------------------------------------------------------------------------------------------------------------
'DLL_API unsigned DLL_CALLCONV FreeImage_GetPitch(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetPitch Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetPitch@4" (ByVal DIB As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Returns the size of the bitmap in bytes. The size of the bitmap is the BITMAPINFOHEADER + the size of the
' palette + the size of the bitmap data.
'-------------------------------------------------------------------------------------------------------------
'DLL_API unsigned DLL_CALLCONV FreeImage_GetDIBSize(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetDIBSize Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetDIBSize@4" (ByVal DIB As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Returns a pointer to the bitmap's palette.  If the bitmap doesn't have a palette, FreeImage_GetPalette
' returns NULL.
'-------------------------------------------------------------------------------------------------------------
'DLL_API RGBQUAD *DLL_CALLCONV FreeImage_GetPalette(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetPalette Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetPalette@4" (ByVal DIB As Long) As Long 'RETURNS RGBQUAD ARRAY... THIS DECLARATION WILL RETURN A POINTER TO THAT ARRAY

'-------------------------------------------------------------------------------------------------------------
' Returns the horizontal resolution, in pixels-per-meter, of the target device for the bitmap.
'-------------------------------------------------------------------------------------------------------------
'DLL_API unsigned DLL_CALLCONV FreeImage_GetDotsPerMeterX(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetDotsPerMeterX Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetDotsPerMeterX@4" (ByVal DIB As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Returns the vertical resolution, in pixels-per-meter, of the target device for the bitmap.
'-------------------------------------------------------------------------------------------------------------
'DLL_API unsigned DLL_CALLCONV FreeImage_GetDotsPerMeterY(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetDotsPerMeterY Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetDotsPerMeterY@4" (ByVal DIB As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Returns a pointer to the bitmap's BITMAPINFOHEADER.
'-------------------------------------------------------------------------------------------------------------
'DLL_API BITMAPINFOHEADER *DLL_CALLCONV FreeImage_GetInfoHeader(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetInfoHeader Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetInfoHeader@4" (ByVal DIB As Long) As Long 'BITMAPINFOHEADER

'-------------------------------------------------------------------------------------------------------------
' Returns a pointer to the bitmap's BITMAPINFO header.
'-------------------------------------------------------------------------------------------------------------
'DLL_API BITMAPINFO *DLL_CALLCONV FreeImage_GetInfo(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetInfo Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetInfo@4" (ByVal DIB As Long) As Long 'BITMAPINFO

'-------------------------------------------------------------------------------------------------------------
' Investigates the colour type of the bitmap and returns one of the following values:
'
' Value            Description
' FIC_MINISWHITE   1-bit bitmap. The min value is white
' FIC_MINISBLACK   1-bit bitmap. The min value is black 8-bit grayscale. The min value is black
' FIC_RGB          24/32-bit RGB
' FIC_PALETTE      1/4/8-bit palletised
'-------------------------------------------------------------------------------------------------------------
'DLL_API FREE_IMAGE_COLOR_TYPE DLL_CALLCONV FreeImage_GetColorType(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_GetColorType Lib "FREEIMAGE.DLL" Alias "_FreeImage_GetColorType@4" (ByVal DIB As Long) As FREE_IMAGE_COLOR_TYPE

'_____________________________________________________________________________________________________________
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
' Conversion routines
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine1To8(BYTE *target, BYTE *source, int width_in_pixels);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine1To8 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine1To8@12" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine4To8(BYTE *target, BYTE *source, int width_in_pixels);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine4To8 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine4To8@12" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine16To8_555(BYTE *target, BYTE *source, int width_in_pixels);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine16To8_555 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine16To8_555@12" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine16To8_565(BYTE *target, BYTE *source, int width_in_pixels);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine16To8_565 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine16To8_565@12" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine24To8(BYTE *target, BYTE *source, int width_in_pixels);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine24To8 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine24To8@12" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine32To8(BYTE *target, BYTE *source, int width_in_pixels);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine32To8 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine32To8@12" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine1To16_555(BYTE *target, BYTE *source, int width_in_pixels, RGBQUAD *palette);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine1To16_555 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine1To16_555@16" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long, ByRef Palette() As RGBQUAD)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine4To16_555(BYTE *target, BYTE *source, int width_in_pixels, RGBQUAD *palette);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine4To16_555 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine4To16_555@16" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long, ByRef Palette() As RGBQUAD)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine8To16_555(BYTE *target, BYTE *source, int width_in_pixels, RGBQUAD *palette);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine8To16_555 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine8To16_555@16" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long, ByRef Palette() As RGBQUAD)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine16_565_To16_555(BYTE *target, BYTE *source, int width_in_pixels);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine16_565_To16_555 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine16_565_To16_555@16" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long, ByRef Palette() As RGBQUAD)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine24To16_555(BYTE *target, BYTE *source, int width_in_pixels);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine24To16_555 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine24To16_555@12" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine32To16_555(BYTE *target, BYTE *source, int width_in_pixels);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine32To16_555 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine32To16_555@12" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine1To16_565(BYTE *target, BYTE *source, int width_in_pixels, RGBQUAD *palette);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine1To16_565 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine1To16_565@16" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long, ByRef Palette() As RGBQUAD)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine4To16_565(BYTE *target, BYTE *source, int width_in_pixels, RGBQUAD *palette);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine4To16_565 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine4To16_565@16" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long, ByRef Palette() As RGBQUAD)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine8To16_565(BYTE *target, BYTE *source, int width_in_pixels, RGBQUAD *palette);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine8To16_565 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine8To16_565@16" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long, ByRef Palette() As RGBQUAD)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine16_555_To16_565(BYTE *target, BYTE *source, int width_in_pixels);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine16_555_To16_565 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine16_555_To16_565@12" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine24To16_565(BYTE *target, BYTE *source, int width_in_pixels);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine24To16_565 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine24To16_565@12" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine32To16_565(BYTE *target, BYTE *source, int width_in_pixels);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine32To16_565 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine32To16_565@12" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine1To24(BYTE *target, BYTE *source, int width_in_pixels, RGBQUAD *palette);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine1To24 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine1To24@16" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long, ByRef Palette() As RGBQUAD)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine4To24(BYTE *target, BYTE *source, int width_in_pixels, RGBQUAD *palette);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine4To24 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine4To24@16" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long, ByRef Palette() As RGBQUAD)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine8To24(BYTE *target, BYTE *source, int width_in_pixels, RGBQUAD *palette);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine8To24 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine8To24@16" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long, ByRef Palette() As RGBQUAD)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine16To24_555(BYTE *target, BYTE *source, int width_in_pixels);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine16To24_555 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine16To24_555@12" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine16To24_565(BYTE *target, BYTE *source, int width_in_pixels);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine16To24_565 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine16To24_565@12" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine32To24(BYTE *target, BYTE *source, int width_in_pixels);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine32To24 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine32To24@12" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine1To32(BYTE *target, BYTE *source, int width_in_pixels, RGBQUAD *palette);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine1To32 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine1To32@16" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long, ByRef Palette() As RGBQUAD)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine4To32(BYTE *target, BYTE *source, int width_in_pixels, RGBQUAD *palette);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine4To32 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine4To32@16" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long, ByRef Palette() As RGBQUAD)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine8To32(BYTE *target, BYTE *source, int width_in_pixels, RGBQUAD *palette);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine8To32 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine8To32@16" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long, ByRef Palette() As RGBQUAD)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine16To32_555(BYTE *target, BYTE *source, int width_in_pixels);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine16To32_555 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine16To32_555@12" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine16To32_565(BYTE *target, BYTE *source, int width_in_pixels);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine16To32_565 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine16To32_565@12" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long)

'-------------------------------------------------------------------------------------------------------------
' Converts a bitmap scanline from the source color depth (bits per pixel) to the target color depth (BPP)
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertLine24To32(BYTE *target, BYTE *source, int width_in_pixels);
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertLine24To32 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertLine24To32@12" (ByRef Target() As Byte, ByRef Source() As Byte, ByVal width_in_pixels As Long)

'-------------------------------------------------------------------------------------------------------------
' Converts the given bitmap to 8 bits. If the bitmap is 24 or 32-bit RGB, the colour values are converted to
' greyscale.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ConvertTo8Bits(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_ConvertTo8Bits Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertTo8Bits@4" (ByVal DIB As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Converts the given bitmap to 16 bits. The resulting bitmap has a layout of 5 bits red, 5 bits green,
' 5 bits red and 1 unused bit.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ConvertTo16Bits555(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_ConvertTo16Bits555 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertTo16Bits555@4" (ByVal DIB As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Converts the given bitmap to 16 bits. The resulting bitmap has a layout of 5 bits red, 6 bits green and 5
' bits red.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ConvertTo16Bits565(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_ConvertTo16Bits565 Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertTo16Bits565@4" (ByVal DIB As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Converts the given bitmap to 24 bits.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ConvertTo24Bits(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_ConvertTo24Bits Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertTo24Bits@4" (ByVal DIB As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Converts the given bitmap to 32 bits.
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ConvertTo32Bits(FIBITMAP *dib);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_ConvertTo32Bits Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertTo32Bits@4" (ByVal DIB As Long) As Long

'-------------------------------------------------------------------------------------------------------------
' Quantizes a full colour 24-bit bitmap to a palletised 8-bit bitmap. The quantize parameter specifies
' which colour reduction algorithm should be used.
'
' Parameter     Description
' FIQ_WUQUANT   Wu's color quantization algorithm
' FIQ_NNQUANT   NeuQuant quantization algorithm
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ColorQuantize(FIBITMAP *dib, FREE_IMAGE_QUANTIZE quantize);
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_ColorQuantize Lib "FREEIMAGE.DLL" Alias "_FreeImage_ColorQuantize@8" (ByVal DIB As Long, ByVal Quantize As FREE_IMAGE_QUANTIZE) As Long

'-------------------------------------------------------------------------------------------------------------
' Grabs a raw piece of memory and converts it into a FreeImage bitmap
'-------------------------------------------------------------------------------------------------------------
'DLL_API FIBITMAP *DLL_CALLCONV FreeImage_ConvertFromRawBits(BYTE *bits, int width, int height, int pitch, unsigned bpp, unsigned red_mask, unsigned green_mask, unsigned blue_mask, BOOL topdown FI_DEFAULT(FALSE));
'-------------------------------------------------------------------------------------------------------------
Public Declare Function FreeImage_ConvertFromRawBits Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertFromRawBits@36" (ByRef Bits() As Byte, ByVal Width As Long, ByVal Height As Long, ByVal Pitch As Long, ByVal BitsPerPixel As Long, ByVal Red_Mask As Long, ByVal Green_Mask As Long, ByVal Blue_Mask As Long, Optional ByVal TopDown As BOOL = FALSE_) As Long

'-------------------------------------------------------------------------------------------------------------
' Writes a freeimage bitmap to a raw piece of memory in a certain format.  Usefull for example to load a
' bitmap into a DirectX surface.
'-------------------------------------------------------------------------------------------------------------
'DLL_API void DLL_CALLCONV FreeImage_ConvertToRawBits(BYTE *bits, FIBITMAP *dib, int pitch, unsigned bpp, unsigned red_mask, unsigned green_mask, unsigned blue_mask, BOOL topdown FI_DEFAULT(FALSE));
'-------------------------------------------------------------------------------------------------------------
Public Declare Sub FreeImage_ConvertToRawBits Lib "FREEIMAGE.DLL" Alias "_FreeImage_ConvertToRawBits@32" (ByRef Bits() As Byte, ByVal DIB As Long, ByVal Pitch As Long, ByVal BitsPerPixel As Long, ByVal Red_Mask As Long, ByVal Green_Mask As Long, ByVal Blue_Mask As Long, Optional ByVal TopDown As BOOL = FALSE_)


'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXX                          WIN32  API  DECLARATIONS                       XXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


' CreateDIBitmap.fdwInit
Public Const CBM_INIT As Long = &H4 ' If this flag is set, the operating system uses the data pointed to by the lpbInit and lpbmi parameters to initialize the bitmaps bits.If this flag is clear, the data pointed to by those parameters is not used.

' SetDIBitsToDevice.ColorUse
Public Const DIB_RGB_COLORS = 0 ' Color table in RGBs
Public Const DIB_PAL_COLORS = 1 ' Color table in palette indices

' SetStretchBltMode.StretchMode
Public Const BLACKONWHITE        As Long = 1 ' Performs a Boolean AND operation using the color values for the eliminated and existing pixels. If the bitmap is a monochrome bitmap, this mode preserves black pixels at the expense of white pixels.
Public Const WHITEONBLACK        As Long = 2 ' Performs a Boolean OR operation using the color values for the eliminated and existing pixels. If the bitmap is a monochrome bitmap, this mode preserves white pixels at the expense of black pixels.
Public Const COLORONCOLOR        As Long = 3 ' Deletes the pixels. This mode deletes all eliminated lines of pixels without trying to preserve their information.
Public Const HALFTONE            As Long = 4 ' Maps pixels from the source rectangle into blocks of pixels in the destination rectangle. The average color over the destination block of pixels approximates the color of the source pixels. After setting the HALFTONE stretching mode, an application must call the SetBrushOrgEx function to set the brush origin. If it fails to do so, brush misalignment occurs.
Public Const STRETCH_ANDSCANS    As Long = BLACKONWHITE ' Same as BLACKONWHITE.
Public Const STRETCH_DELETESCANS As Long = COLORONCOLOR ' Same as COLORONCOLOR.
Public Const STRETCH_HALFTONE    As Long = HALFTONE     ' Same as HALFTONE.
Public Const STRETCH_ORSCANS     As Long = WHITEONBLACK ' Same as WHITEONBLACK.

' GetObjectType (Return)
Public Const OBJ_PEN         As Long = 1  ' Pen
Public Const OBJ_BRUSH       As Long = 2  ' Brush
Public Const OBJ_DC          As Long = 3  ' Device context
Public Const OBJ_METADC      As Long = 4  ' Metafile device context
Public Const OBJ_PAL         As Long = 5  ' Palette
Public Const OBJ_FONT        As Long = 6  ' Font
Public Const OBJ_BITMAP      As Long = 7  ' BITMAP
Public Const OBJ_REGION      As Long = 8  ' Region
Public Const OBJ_METAFILE    As Long = 9  ' metafile
Public Const OBJ_MEMDC       As Long = 10 ' Memory device context
Public Const OBJ_EXTPEN      As Long = 11 ' Extended pen
Public Const OBJ_ENHMETADC   As Long = 12 ' Enhanced metafile device context
Public Const OBJ_ENHMETAFILE As Long = 13 ' Enhanced metafile
Public Const OBJ_COLORSPACE  As Long = 14 ' Color Space

' Win32 API Sub/Function Declarations
Public Declare Sub CopyMemory Lib "KERNEL32.DLL" Alias "RtlMoveMemory" (Destination As Any, Source As Any, ByVal Length As Long)
Public Declare Function BitBlt Lib "GDI32.DLL" (ByVal Dest_hDC As Long, ByVal Dest_X As Long, ByVal Dest_Y As Long, ByVal Dest_Width As Long, ByVal Dest_Height As Long, ByVal Src_hDC As Long, ByVal Src_X As Long, ByVal Src_Y As Long, ByVal RasterOperation As RasterOpConstants) As BOOL
Public Declare Function CreateCompatibleDC Lib "GDI32.DLL" (ByVal hDC As Long) As Long
Public Declare Function CreateDIBitmap Lib "GDI32.DLL" (ByVal hDC As Long, ByRef lpBITMAPINFOHEADER As Any, ByVal BitFlags As Long, ByRef lpBits As Any, ByRef lpBitmapInfo As Any, ByVal fuUsage As Long) As Long
Public Declare Function DeleteDC Lib "GDI32.DLL" (ByVal hDC As Long) As BOOL
Public Declare Function DeleteObject Lib "GDI32.DLL" (ByVal hObject As Long) As BOOL
Public Declare Function GetDC Lib "USER32.DLL" (ByVal hwnd As Long) As Long
Public Declare Function GetDesktopWindow Lib "USER32.DLL" () As Long
Public Declare Function GetDIBits Lib "gdi32" (ByVal hDC As Long, ByVal hBitmap As Long, ByVal StartingScanLine As Long, ByVal ScanLineCount As Long, ByRef lpBits As Any, ByRef lpBitmapInfo As Any, ByVal ColorUsage As Long) As Long
Public Declare Function GetGdiObject Lib "GDI32.DLL" Alias "GetObjectA" (ByVal hObject As Long, ByVal BufferLength As Long, ByRef ObjectInfo As Any) As Long
Public Declare Function GetObjectType Lib "GDI32.DLL" (ByVal hObject As Long) As Long
Public Declare Function ReleaseDC Lib "USER32.DLL" (ByVal hwnd As Long, ByVal hDC As Long) As Long
Public Declare Function SelectObject Lib "GDI32.DLL" (ByVal hDC As Long, ByVal hObject As Long) As Long
Public Declare Function SetDIBitsToDevice Lib "GDI32.DLL" (ByVal Dest_hDC As Long, ByVal Dest_X As Long, ByVal Dest_Y As Long, ByVal Src_Width As Long, ByVal Src_Height As Long, ByVal Src_X As Long, ByVal Src_Y As Long, ByVal StartingScanLine As Long, ByVal ScanLineCount As Long, ByRef lpBits As Any, ByRef lpBMI As Any, ByVal ColorUse As Long) As Long
Public Declare Function SetStretchBltMode Lib "GDI32.DLL" (ByVal hDC As Long, ByVal StretchMode As Long) As Long
Public Declare Function StretchBlt Lib "GDI32.DLL" (ByVal Dest_hDC As Long, ByVal Dest_X As Long, ByVal Dest_Y As Long, ByVal Dest_Width As Long, ByVal Dest_Height As Long, ByVal Src_hDC As Long, ByVal Src_X As Long, ByVal Src_Y As Long, ByVal Src_Width As Long, ByVal Src_Height As Long, ByVal RasterOperation As RasterOpConstants) As BOOL
Public Declare Function GetCurrentObject Lib "gdi32" (ByVal hDC As Long, ByVal uObjectType As Long) As Long


'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXX                    SUPPORTING FUNCTION  DECLARATIONS                    XXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


'-------------------------------------------------------------------------------------------------------------
' Callback function that receives status/error messages generated inside the DLL. If a bitmap load action
' fails for example, this function receives why it failed.
'-------------------------------------------------------------------------------------------------------------
'typedef void (*FreeImage_OutputMessageFunction)(FREE_IMAGE_FORMAT fif, const char *msg);
'-------------------------------------------------------------------------------------------------------------
Public Sub FI_OutputMessageProc(ByVal ImageFormat As FREE_IMAGE_FORMAT, ByVal strMessage As String)
  
  Debug.Print "FI_OutputMessageProc - ImageFormat = " & CStr(ImageFormat) & ", Message = " & strMessage
  
End Sub

'typedef unsigned (DLL_CALLCONV *FI_ReadProc) (void *buffer, unsigned size, unsigned count, fi_handle handle);
Public Function FI_ReadProc(ByVal Buffer As Long, ByVal Size As Long, ByVal Count As Long, ByVal Handle As Long) As Long
  
  'return fread(buffer, s, c, (FILE *)handle);
  
End Function

'typedef unsigned (DLL_CALLCONV *FI_WriteProc) (void *buffer, unsigned size, unsigned count, fi_handle handle);
Public Function FI_WriteProc(ByVal Buffer As Long, ByVal Size As Long, ByVal Count As Long, ByVal Handle As Long) As Long
  
  'return fwrite(buffer, s, c, (FILE *)handle);
  
End Function

'typedef int (DLL_CALLCONV *FI_SeekProc) (fi_handle handle, long offset, int origin);
Public Function FI_SeekProc(ByVal Handle As Long, ByVal Offset As Long, ByVal Origin As Long) As Long
  
  'return fseek((FILE *)handle, offset, origin);
  
End Function

'typedef long (DLL_CALLCONV *FI_TellProc) (fi_handle handle);
Public Function FI_TellProc(ByVal Handle As Long) As Long
  
  'return ftell((FILE *)handle);
  
End Function


'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXX                       CUSTOM  FUNCTION  DECLARATIONS                    XXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


'=============================================================================================================
' FreeImage_BMP_From_DIB
'
' This function takes a Device Independant Bitmap (DIB) returned from FreeImage and converts it to a standard
' Win32 BITMAP.
'
' NOTE: The caller of this function is responsbile for cleaning up the returned BITMAP GDI object (DeleteObject)
'
' Parameter:                    Use:
' 
' DIB                           Specifies the Device Independant Bitmap (DIB) to convert to a BITMAP.  This
'                               is usually passed in as the result of a call to a FreeImage_* function.
' Return_hBITMAP                Returns a handle to a valid Win32 GDI BITMAP object.
'
' Return:
' 
' Returns TRUE if the function executed successfully.
' Returns FALSE if the function failed to execute successfully.
'=============================================================================================================
Public Function FreeImage_BMP_From_DIB(ByVal DIB As Long, ByRef Return_hBITMAP As Long) As Boolean
  
  Dim hDC_Screen As Long
  Dim lpBMI      As Long
  Dim lpBMIH     As Long
  Dim lpBits     As Long
  
  ' Clear return values
  Return_hBITMAP = 0
  
  ' Validate parameters
  If DIB = 0 Then Exit Function
  
  ' Get the required information from the DIB
  lpBMIH = FreeImage_GetInfoHeader(DIB)
  If lpBMIH = 0 Then Exit Function
  lpBMI = FreeImage_GetInfo(DIB)
  If lpBMI = 0 Then Exit Function
  lpBits = FreeImage_GetBits(DIB)
  If lpBits = 0 Then Exit Function
  
  ' Get a reference to the current display's Device Context (DC)
  hDC_Screen = GetDC(GetDesktopWindow)
  If hDC_Screen = 0 Then Exit Function
  
  ' Create a new BITMAP from the specified DIB
  Return_hBITMAP = CreateDIBitmap(hDC_Screen, ByVal lpBMIH, CBM_INIT, ByVal lpBits, ByVal lpBMI, DIB_RGB_COLORS)
  If Return_hBITMAP <> 0 Then FreeImage_BMP_From_DIB = True
  
CleanUp:
  ReleaseDC GetDesktopWindow, hDC_Screen
  
End Function

'=============================================================================================================
' FreeImage_DIB_From_BMP
'
' This function takes a standard Win32 BITMAP and converts it to a Device Independant Bitmap (DIB) that can
' be used with FreeImage.
'
' NOTE: The caller of this function is responsbile for cleaning up the returned BITMAP GDI object (DeleteObject)
'
' Parameter:                    Use:
' 
' hBITMAP                       Specifies the handle to a valid Win32 GDI BITMAP object.
' Return_DIB                    Returns the handle to the Device Independant Bitmap (DIB) that was created
'                               from the specified BITMAP.  This can be used with FreeImage_* functions.
'
' Return:
' 
' Returns TRUE if the function executed successfully.
' Returns FALSE if the function failed to execute successfully.
'=============================================================================================================
Public Function FreeImage_DIB_From_BMP(ByVal hBitmap As Long, ByRef Return_DIB As Long) As Boolean
  
  Dim hDC_Screen As Long
  Dim TempDIB    As Long
  Dim lpBits     As Long
  Dim lpBMI      As Long
  Dim TheWidth   As Long
  Dim TheHeight  As Long
  Dim TheBPP     As Integer
  
  ' Clear return values
  Return_DIB = 0
  
  ' Validate the BITMAP and get the information we need from it
  If FreeImage_GetBitmapInfo(hBitmap, TheWidth, TheHeight) = False Then Exit Function ', , TheBPP) = False Then Exit Function
  
  ' This is hard coded as a "work around" to a problem in getting the correct color depth of images returned by FreeImage
  TheBPP = 32
  
  ' Get a reference to the current display's Device Context (DC)
  hDC_Screen = GetDC(GetDesktopWindow)
  If hDC_Screen = 0 Then Exit Function
  
  ' Allocate a DIB and get the information from it we need
  TempDIB = FreeImage_Allocate(TheWidth, TheHeight, TheBPP)
  If TempDIB = 0 Then GoTo CleanUp
  lpBMI = FreeImage_GetInfo(TempDIB)
  If lpBMI = 0 Then Call FreeImage_Free(TempDIB): GoTo CleanUp
  lpBits = FreeImage_GetBits(TempDIB)
  If lpBits = 0 Then Call FreeImage_Free(TempDIB): GoTo CleanUp
  
  ' Convert the DIB to a BITMAP
  If GetDIBits(hDC_Screen, hBitmap, 0, TheHeight, ByVal lpBits, ByVal lpBMI, DIB_RGB_COLORS) <> 0 Then
    FreeImage_DIB_From_BMP = True
    Return_DIB = TempDIB
  Else
    FreeImage_Free TempDIB
  End If
  
CleanUp:
  ReleaseDC GetDesktopWindow, hDC_Screen
  
End Function

'=============================================================================================================
' FreeImage_RenderDIB
'
' This function takes a Device Indpendant Bitmap (DIB) returned from FreeImage and renders it to the specified
' Device Context (DC).
'
' NOTE: This function can be used to render a DIB onto a memory Device Context (DC) or a window-based DC
'
' Parameter:                    Use:
' 
' DIB                           Specifies the Device Independant Bitmap to render into the specified DC
' Dest_hDC                      Specifies the memory based DC (Device Context) or window based DC to render onto
' Dest_X                        Optioanl. Specifies the X coordinate (Left) of where the image should be rendered
' Dest_Y                        Optioanl. Specifies the Y coordinate (Top) of where the image should be rendered
' Src_Width                     Optioanl. Specifies the width of the DIB to render.  If this isn't specified,
'                               this function attempts to get the width from the image passed in.
' Src_Height                    Optioanl. Specifies the height of the DIB to render.  If this isn't specified,
'                               this function attempts to get the height from the image passed in.
' Return_ErrNum                 Optioanl. If an error occurs, this returns the Win32 error number.
'
' Return:
' 
' Returns TRUE if the function executed successfully.
' Returns FALSE if the function failed to execute successfully.
'=============================================================================================================
Public Function FreeImage_RenderDIB(ByVal DIB As Long, ByVal Dest_hDC As Long, Optional ByVal Dest_X As Long = 0, Optional ByVal Dest_Y As Long = 0, Optional ByVal Src_Width As Long = -1, Optional ByVal Src_Height As Long = -1, Optional ByRef Return_ErrNum As Long) As Boolean
  
  Dim lpBMI     As Long
  Dim lpBits    As Long
  Dim TheWidth  As Long
  Dim TheHeight As Long
  
  ' Clear return values
  Return_ErrNum = 0
  
  ' Validate parameters
  If DIB = 0 Then
    Return_ErrNum = 1: Exit Function
  ElseIf GetObjectType(Dest_hDC) <> OBJ_DC And GetObjectType(Dest_hDC) <> OBJ_MEMDC Then
    Return_ErrNum = 2: Exit Function
  End If
  
  ' Get the width to use
  If Src_Width > 0 Then
    TheWidth = Src_Width
  Else
    TheWidth = FreeImage_GetWidth(DIB)
    If TheWidth < 1 Then Return_ErrNum = 3: Exit Function
  End If
  
  ' Get the height to use
  If Src_Height > 0 Then
    TheHeight = Src_Height
  Else
    TheHeight = FreeImage_GetHeight(DIB)
    If TheHeight < 1 Then Return_ErrNum = 4: Exit Function
  End If
  
  ' Get the bits of the DIB
  lpBits = FreeImage_GetBits(DIB)
  If lpBits = 0 Then Return_ErrNum = 5: Exit Function
  
  ' Get the BITMAP info for the DIB
  lpBMI = FreeImage_GetInfo(DIB)
  If lpBMI = 0 Then Return_ErrNum = 6: Exit Function
  
  ' Render the DIB to the specified device
  If SetDIBitsToDevice(Dest_hDC, Dest_X, Dest_Y, TheWidth, TheHeight, 0, 0, 0, TheHeight, ByVal lpBits, ByVal lpBMI, DIB_RGB_COLORS) = 0 Then
    Return_ErrNum = Err.LastDllError
  Else
    FreeImage_RenderDIB = True
  End If
  
End Function

'=============================================================================================================
' FreeImage_RenderBitmap
'
' This function takes a standard Win32 BITMAP and renders it to the specified Device Context (DC).
'
' NOTE: This function can be used to render a BITMAP onto a memory Device Context (DC) or a window-based DC
'
' Parameter:                    Use:
' 
' hBITMAP                       Specifies the Win32 GDI BITMAP object to render into the specified DC
' Dest_hDC                      Specifies the memory based DC (Device Context) or window based DC to render onto
' Dest_X                        Optioanl. Specifies the X coordinate (Left) of where the image should be rendered
' Dest_Y                        Optioanl. Specifies the Y coordinate (Top) of where the image should be rendered
' Src_Width                     Optioanl. Specifies the width of the BITMAP to render.  If this isn't specified,
'                               this function attempts to get the width from the image passed in.
' Src_Height                    Optioanl. Specifies the height of the BITMAP to render.  If this isn't specified,
'                               this function attempts to get the height from the image passed in.
' blnInvertBitmapColors         Optional. If set to TRUE, the colors of the specified image will be inverted.
' Return_ErrNum                 Optioanl. If an error occurs, this returns the Win32 error number.
'
' Return:
' 
' Returns TRUE if the function executed successfully.
' Returns FALSE if the function failed to execute successfully.
'=============================================================================================================
Public Function FreeImage_RenderBitmap(ByVal hBitmap As Long, ByVal Dest_hDC As Long, Optional ByVal Dest_X As Long = 0, Optional ByVal Dest_Y As Long = 0, Optional ByVal Src_Width As Long = -1, Optional ByVal Src_Height As Long = -1, Optional ByVal blnInvertBitmapColors As Boolean = False, Optional ByRef Return_ErrNum As Long) As Boolean
  
  Dim hDC       As Long
  Dim hBMP_Prev As Long
  Dim TheWidth  As Long
  Dim TheHeight As Long
  Dim TheFlags  As Long
  
  ' Clear return values
  Return_ErrNum = 0
  
  ' Validate parameters
  If hBitmap = 0 Then
    Return_ErrNum = 1: Exit Function
  ElseIf GetObjectType(hBitmap) <> OBJ_BITMAP Then
    Return_ErrNum = 2: Exit Function
  ElseIf GetObjectType(Dest_hDC) <> OBJ_DC And GetObjectType(Dest_hDC) <> OBJ_MEMDC Then
    Return_ErrNum = 3: Exit Function
  End If
  
  ' Get the width to use
  If Src_Width > 0 Then
    TheWidth = Src_Width
  Else
    TheWidth = FreeImage_GetBitmapWidth(hBitmap)
    If TheWidth < 1 Then Return_ErrNum = 4: Exit Function
  End If
  
  ' Get the height to use
  If Src_Height > 0 Then
    TheHeight = Src_Height
  Else
    TheHeight = FreeImage_GetBitmapHeight(hBitmap)
    If TheHeight < 1 Then Return_ErrNum = 5: Exit Function
  End If
  
  ' Create a DC to draw with
  If MemoryDC_Create(hDC) = False Then
    Return_ErrNum = 6: Exit Function
  End If
  
  ' Put the BITMAP into the newly created memory DC
  hBMP_Prev = SelectObject(hDC, hBitmap)
  
  ' Draw onto the specified DC
  TheFlags = vbSrcCopy
  If blnInvertBitmapColors = True Then TheFlags = vbNotSrcCopy
  If BitBlt(Dest_hDC, Dest_X, Dest_Y, TheWidth, TheHeight, hDC, 0, 0, TheFlags) = FALSE_ Then
    Return_ErrNum = Err.LastDllError
  Else
    FreeImage_RenderBitmap = True
  End If
  
  ' Take our bitmap out of the DC
  SelectObject hDC, hBMP_Prev
  
  ' Clean up the tempoary memory DC
  MemoryDC_Destroy hDC
  
End Function

'=============================================================================================================
' FreeImage_GetBitmapWidth
'
' This function returns the width of the specified Win32 BITMAP in pixels.
'
' Parameter:                    Use:
' 
' hBITMAP                       Specifies the Win32 GDI BITMAP object to get the information from
'
' Return:
' 
' If successful, returns the width in pixels of the specified BITMAP
' If failed, returns ZERIO (0)
'=============================================================================================================
Public Function FreeImage_GetBitmapWidth(ByVal hBitmap As Long) As Long
  
  Dim BmpInfo As BITMAP
  
  ' Validate parameters
  If hBitmap = 0 Then Exit Function
  If GetObjectType(hBitmap) <> OBJ_BITMAP Then Exit Function
  
  ' Get the information for the bitmap
  If GetGdiObject(hBitmap, Len(BmpInfo), BmpInfo) = 0 Then Exit Function
  FreeImage_GetBitmapWidth = BmpInfo.bmWidth
  
End Function

'=============================================================================================================
' FreeImage_GetBitmapHeight
'
' This function returns the height of the specified Win32 BITMAP in pixels.
'
' Parameter:                    Use:
' 
' hBITMAP                       Specifies the Win32 GDI BITMAP object to get the information from
'
' Return:
' 
' If successful, returns the width in pixels of the specified BITMAP
' If failed, returns ZERIO (0)
'=============================================================================================================
Public Function FreeImage_GetBitmapHeight(ByVal hBitmap As Long) As Long
  
  Dim BmpInfo As BITMAP
  
  ' Validate parameters
  If hBitmap = 0 Then Exit Function
  If GetObjectType(hBitmap) <> OBJ_BITMAP Then Exit Function
  
  ' Get the information for the bitmap
  If GetGdiObject(hBitmap, Len(BmpInfo), BmpInfo) = 0 Then Exit Function
  FreeImage_GetBitmapHeight = BmpInfo.bmHeight
  
End Function

'=============================================================================================================
' FreeImage_GetBitmapInfo
'
' This function returns the information of the specified Win32 BITMAP.
'
' Parameter:                    Use:
' 
' hBITMAP                       Specifies the valid Win32 BITMAP object to get information about.
' Return_Width                  Optional. Returns the width of the specified BITMAP in pixels.
' Return_Height                 Optional. Returns the height of the specified BITMAP in pixels.
' Return_Type                   Optional. Returns the type of BITMAP specified.
' Return_BitsPerPixel           Optional. Returns the color depth (bits per pixel) of the specified BITMAP.
'                               1  = Black and white
'                               4  = 16 Colors
'                               8  = 256 Colors
'                               16 = 16 bit Color (65536 Colors)
'                               24 = 24 bit Color - True Color (16777216 Colors)
'                               32 = 32 bit Color - High Color (4294967296 Colors)
' Return_Planes                 Optional. Specifies the count of color planes.  This is usually 1.
' Return_PointerToBits          Optional. Returns a pointer to the memory location of the first bit of the
'                               bit array that makes up the colors of the specified BITMAP.
'
' Return:
' 
' Returns TRUE if the function executed successfully.
' Returns FALSE if the function failed to execute successfully.
'=============================================================================================================
Public Function FreeImage_GetBitmapInfo(ByVal hBitmap As Long, Optional ByRef Return_Width As Long, Optional ByRef Return_Height As Long, Optional ByRef Return_Type As Long, Optional ByRef Return_BitsPerPixel As Integer, Optional ByRef Return_Planes As Integer, Optional ByRef Return_PointerToBits As Long) As Boolean
  
  Dim BmpInfo As BITMAP
  
  ' Clear return values
  Return_BitsPerPixel = -1
  Return_Height = -1
  Return_Planes = -1
  Return_PointerToBits = -1
  Return_Type = -1
  Return_Width = -1
  
  ' Validate parameters
  If hBitmap = 0 Then Exit Function
  If GetObjectType(hBitmap) <> OBJ_BITMAP Then Exit Function
  
  ' Get the information for the bitmap
  If GetGdiObject(hBitmap, Len(BmpInfo), BmpInfo) = 0 Then Exit Function
  
  ' Return the information
  With BmpInfo
    Return_Height = .bmHeight
    Return_Planes = .bmPlanes
    Return_Type = .bmType
    Return_Width = .bmWidth
    Return_BitsPerPixel = .bmPlanes * .bmBitsPixel
    Return_PointerToBits = .bmBits
  End With
  FreeImage_GetBitmapInfo = True
  
End Function

'=============================================================================================================
' FreeImage_LoadMultiPage
'
' This function takes the specified file and returns back an array of BITMAP handles representing the
' BITMAP(s) that make it up
'
' NOTE: The caller is responsible for destroying the returned BITMAP(s) by calling the Win32 DeleteObject API
'
' Parameter:                    Use:
' 
' FileName                      File name or file path to the Multi-Page image file you wish to open
' Return_HBITMAPs               Returns an array of handles to the BITMAPS(s) that make up the original
'                               Multi-Page image.
' Return_DibCount               Returns the number of images found within the specified image file
'
' Return:
' 
' Returns TRUE if the function executed successfully.
' Returns FALSE if the function failed to execute successfully.
'=============================================================================================================
Public Function FreeImage_LoadMultiPage(ByVal FileName As String, ByRef Return_HBITMAPs() As Long, ByRef Return_BitmapCount As Long) As Boolean
  
  Dim ImageFormat As FREE_IMAGE_FORMAT
  Dim MBMP        As Long
  Dim Page        As Long
  Dim PageCount   As Long
  Dim hBitmapTemp As Long
  Dim lngCounter  As Long
  
  ' Clear return values
  Erase Return_HBITMAPs
  Return_BitmapCount = 0
  
  ' Validate parameters
  FileName = Trim(FileName)
  If FileName = "" Then Exit Function
  If Dir(FileName, vbArchive Or vbHidden Or vbNormal Or vbReadOnly Or vbSystem) = "" Then Exit Function
  ImageFormat = FreeImage_GetFileType(FileName, FileLen(FileName))
  
  ' Open the file as a Multi-Page
  MBMP = FreeImage_OpenMultiBitmap(ImageFormat, FileName, FALSE_, TRUE_, FALSE_)
  If MBMP = 0 Then Exit Function
  
  ' Get the total number of pages in the image
  PageCount = FreeImage_GetPageCount(MBMP)
  If PageCount < 1 Then GoTo CleanUp
  
  ' Loop through the pages and transfer the DIB pages returned into BITMAPS and return them as an array of HANDLE(s)
  For lngCounter = 0 To PageCount - 1
    
    ' Lock the current page to work with it
    Page = FreeImage_LockPage(MBMP, lngCounter)
    
    ' Make sure the specified page is valid
    If Page <> 0 Then
      
      ' Convert the current DIB image to a BITMAP and add it to the returned array
      If FreeImage_BMP_From_DIB(Page, hBitmapTemp) = True Then
        If hBitmapTemp <> 0 Then
          ReDim Preserve Return_HBITMAPs(0 To Return_BitmapCount) As Long
          Return_HBITMAPs(Return_BitmapCount) = hBitmapTemp
          Return_BitmapCount = Return_BitmapCount + 1
        End If
      End If
      
      ' Unlock the current page
      FreeImage_UnlockPage MBMP, Page, FALSE_
    End If
    
  Next
  
  FreeImage_LoadMultiPage = True
  
CleanUp:
  
  ' Close the multibitmap
  FreeImage_CloseMultiBitmap MBMP
  
End Function

'=============================================================================================================
' FreeImage_LoadMultiPageEx
'
' This function takes the specified file and returns back an array of DIB handles representing the Device
' Independant BITMAP(s) that make it up.
'
' NOTE: The caller is responsible for destroying the returned DIB(s) by calling the "FreeImage_Free" or
' "FreeImage_Unload" API
'
' Parameter:                    Use:
' 
' FileName                      File name or file path to the Multi-Page image file you wish to open
' Return_DIBs                   Returns an array of handles to the Device Independant Bitamp(s) that make up
'                               the original Multi-Page image.
' Return_DibCount               Returns the number of images found within the specified image file
'
' Return:
' 
' Returns TRUE if the function executed successfully.
' Returns FALSE if the function failed to execute successfully.
'=============================================================================================================
Public Function FreeImage_LoadMultiPageEx(ByVal FileName As String, ByRef Return_DIBs() As Long, ByRef Return_DibCount As Long) As Boolean
  
  Dim ImageFormat As FREE_IMAGE_FORMAT
  Dim MBMP        As Long
  Dim Page        As Long
  Dim PageCount   As Long
  Dim hDibTemp    As Long
  Dim lngCounter  As Long
  
  ' Clear return values
  Erase Return_DIBs
  Return_DibCount = 0
  
  ' Validate parameters
  FileName = Trim(FileName)
  If FileName = "" Then Exit Function
  If Dir(FileName, vbArchive Or vbHidden Or vbNormal Or vbReadOnly Or vbSystem) = "" Then Exit Function
  ImageFormat = FreeImage_GetFileType(FileName, FileLen(FileName))
  
  ' Open the file as a Multi-Page
  MBMP = FreeImage_OpenMultiBitmap(ImageFormat, FileName, FALSE_, TRUE_, FALSE_)
  If MBMP = 0 Then Exit Function
  
  ' Get the total number of pages in the image
  PageCount = FreeImage_GetPageCount(MBMP)
  If PageCount < 1 Then GoTo CleanUp
  
  ' Loop through the pages and transfer the DIB pages returned into BITMAPS and return them as an array of HANDLE(s)
  For lngCounter = 0 To PageCount - 1
    
    ' Lock the current page to work with it
    Page = FreeImage_LockPage(MBMP, lngCounter)
    
    ' Make sure the specified page is valid
    If Page <> 0 Then
      
      ' Make a copy of the page and return it in the array
      hDibTemp = FreeImage_Clone(Page)
      If hDibTemp <> 0 Then
        ReDim Preserve Return_DIBs(0 To Return_DibCount) As Long
        Return_DIBs(Return_DibCount) = hDibTemp
        Return_DibCount = Return_DibCount + 1
      End If
      
      ' Unlock the current page
      FreeImage_UnlockPage MBMP, Page, FALSE_
    End If
    
  Next
  
  FreeImage_LoadMultiPageEx = True
  
CleanUp:
  
  ' Close the multibitmap
  FreeImage_CloseMultiBitmap MBMP
  
End Function

'=============================================================================================================
' BITMAPINFO_From_Ptr
'
' This function takes the specified pointer to a BITMAPINFO structure and returns that structure
'
' Parameter:                    Use:
' 
' lngPointer                    A long value that represents a pointer to the BITMAPINFO information to get
' Return_BITMAPINFO             Returns the BITMAPINFO information (if the pointer is a valid pointer to it)
'
' Return:
' 
' Nothing
'=============================================================================================================
Public Sub BITMAPINFO_From_Ptr(ByVal lngPointer As Long, ByRef Return_BITMAPINFO As BITMAPINFO)
  
  ' Validate parameters
  If lngPointer = 0 Then Exit Sub
  
  ' Copy the memory from the pointer to the structure
  CopyMemory Return_BITMAPINFO, ByVal lngPointer, Len(Return_BITMAPINFO)
  
End Sub

'=============================================================================================================
' BITMAPINFOHEADER_From_Ptr
'
' This function takes the specified pointer to a BITMAPINFOHEADER structure and returns that structure
'
' Parameter:                    Use:
' 
' lngPointer                    A long value that represents a pointer to the BITMAPINFOHEADER information to get
' Return_BITMAPINFOHEADER       Returns the BITMAPINFOHEADER information (if the pointer is a valid pointer to it)
'
' Return:
' 
' Nothing
'=============================================================================================================
Public Sub BITMAPINFOHEADER_From_Ptr(ByVal lngPointer As Long, ByRef Return_BITMAPINFOHEADER As BITMAPINFOHEADER)
  
  ' Validate parameters
  If lngPointer = 0 Then Exit Sub
  
  ' Copy the memory from the pointer to the structure
  CopyMemory Return_BITMAPINFOHEADER, ByVal lngPointer, Len(Return_BITMAPINFOHEADER)
  
End Sub


'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXX            HELPER  FUNCTION  DECLARATIONS  (USED LOCALLY ONLY)          XXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


' This function creates a memory Device Context (DC) compatible with the user's current display.  This DC can be used to perform BITMAP opperations.
Private Function MemoryDC_Create(ByRef Return_hDC As Long) As Boolean
  
  Dim hDC_Screen  As Long
  
  ' Clear return value
  Return_hDC = 0
  
  ' Get a reference to the current display's DC
  hDC_Screen = GetDC(GetDesktopWindow)
  If hDC_Screen = 0 Then Exit Function
  
  ' Create a DC that is compatible with the current display
  Return_hDC = CreateCompatibleDC(hDC_Screen)
  If Return_hDC <> 0 Then MemoryDC_Create = True
  
  ' Release the reference to the display's DC
  ReleaseDC GetDesktopWindow, hDC_Screen
  
End Function

' This function takes the specified memory Device Context (DC) and deletes it and any bitmaps that are currently selected into it.
Private Function MemoryDC_Destroy(ByRef hDC As Long, Optional ByRef hBITMAP_Previous As Long) As Boolean
  
  ' If there's a previous BITMAP, put it back before destroying the DC
  If hBITMAP_Previous <> 0 Then
    DeleteObject SelectObject(hDC, hBITMAP_Previous)
    hBITMAP_Previous = 0
  End If
  
  ' Delete the DC and remove the reference
  If DeleteDC(hDC) <> FALSE_ Then
    hDC = 0
    MemoryDC_Destroy = True
  End If
  
End Function

Public Function GetBitmapFromDC(ByVal hDC As Long) As Long
  
  GetBitmapFromDC = GetCurrentObject(hDC, OBJ_BITMAP)
  
End Function
